import inspect
import logging
import time

from biliup.config import config
from .engine.decorators import Plugin

logger = logging.getLogger('biliup')


def upload(data):
    """
    上传入口
    :param platform:
    :param index:
    :param data: 现在需包含内容{url,date} 完整包含内容{url,date,format_title}
    :return:
    """
    try:
        index = data['name']
        context = {**config, **config['streamers'][index]}
        platform = context.get("uploader") if context.get("uploader") else "biliup-rs"
        cls = Plugin.upload_plugins.get(platform)
        if cls is None:
            return logger.error(f"No such uploader: {platform}")
        data, context = fmt_title_and_desc(data)
        data['dolby'] = config.get('dolby', 0)
        data['hires'] = config.get('hires', 0)
        data['no_reprint'] = config.get('no_reprint', 0)
        data['open_elec'] = config.get('open_elec', 0)
        sig = inspect.signature(cls)
        kwargs = {}
        for k in sig.parameters:
            v = context.get(k)
            if v:
                kwargs[k] = v
        return cls(index, data, **kwargs).start()
    except:
        logger.exception("Uncaught exception:")


def biliup_uploader(filelist, data):
    try:
        index = data['name']
        context = {**data}
        platform = context.get("uploader") if context.get("uploader") else "biliup-rs"
        cls = Plugin.upload_plugins.get(platform)
        if cls is None:
            return logger.error(f"No such uploader: {platform}")
        data, context = fmt_title_and_desc_m(data)
        data['dolby'] = data.get('dolby', 0)
        data['hires'] = data.get('hires', 0)
        data['no_reprint'] = data.get('no_reprint', 0)
        data['open_elec'] = data.get('open_elec', 0)
        sig = inspect.signature(cls)
        kwargs = {}
        for k in sig.parameters:
            v = context.get(k)
            if v:
                kwargs[k] = v
        logger.info("start biliup")
        return cls(index, data, **kwargs).upload(filelist)
    except:
        logger.exception("Uncaught exception:")
    else:
        logger.info("stop biliup")


def fmt_title_and_desc_m(data):
    index = data['name']
    context = {**data}
    streamer = data.get('streamer', index)
    date = data.get("date", time.localtime())
    title = data.get('title', index)
    url = data.get('url')
    data["format_title"] = custom_fmtstr(context.get('title') or f'%Y.%m.%d{index}', date, title, streamer, url)
    if context.get('description'):
        context['description'] = custom_fmtstr(context.get('description'), date, title, streamer, url)
    return data, context


# 将格式化标题和简介拆分出来方便复用
def fmt_title_and_desc(data):
    index = data['name']
    context = {**config, **config['streamers'][index]}
    streamer = data.get('streamer', index)
    date = data.get("date", time.localtime())
    title = data.get('title', index)
    url = data.get('url')
    data["format_title"] = custom_fmtstr(context.get('title') or f'%Y.%m.%d{index}', date, title, streamer, url)
    if context.get('description'):
        context['description'] = custom_fmtstr(context.get('description'), date, title, streamer, url)
    return data, context


def custom_fmtstr(string, date, title, streamer, url):
    return time.strftime(string.encode('unicode-escape').decode(), date).encode().decode("unicode-escape").format(
        title=title, streamer=streamer, url=url)



import json

import aiohttp

from biliup.plugins import random_user_agent


class Twitcasting:
    heartbeat = None
    fake_headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate, br",
        "Cache-Control": "no-cache",
        "Pragma": "no-cache",
        "Referer": "https://twitcasting.tv/",
        "User-Agent": random_user_agent()
    }

    @staticmethod
    async def get_ws_info(url, context):
        async with aiohttp.ClientSession(headers=Twitcasting.fake_headers) as session:
            async with session.post(
                    url="https://twitcasting.tv/eventpubsuburl.php",
                    data={
                        'movie_id': context['movie_id'],
                        'password': context['password']
                    },
                    timeout=5
            ) as resp:
                r_obj = await resp.json()
                url = r_obj['url']
                return url, []

    @staticmethod
    def decode_msg(data):
        msgs = []
        if data is not None:
            for d in data.splitlines():
                if len(d) == 0:
                    continue
                try:
                    d = json.loads(d)[0]
                    msg = {
                        "content": d['message'],
                        "msg_type": "danmaku"
                    }
                    msgs.append(msg)
                except:
                    pass
        return msgs



import logging
import os
import pathlib
import shutil

from typing import NamedTuple, Optional, List

from sqlalchemy import desc

from biliup.common.tools import NamedLock, get_file_create_timestamp
from biliup.config import config
from biliup.database import models
from biliup.database.db import SessionLocal

logger = logging.getLogger('biliup')


class UploadBase:
    class FileInfo(NamedTuple):
        video: str
        danmaku: Optional[str]

    def __init__(self, principal, data, persistence_path=None, postprocessor=None):
        self.principal = principal
        self.persistence_path = persistence_path
        self.data: dict = data
        self.post_processor = postprocessor

    @staticmethod
    def file_list(index) -> List[FileInfo]:
        from biliup.handler import event_manager
        media_extensions = ['.mp4', '.flv', '.3gp', '.webm', '.mkv', '.ts']

        # 获取文件列表
        file_list = []
        # 数据库中保存的文件名, 不含后缀
        save = []
        with SessionLocal() as db:
            dbinfo = db.query(models.StreamerInfo).filter(models.StreamerInfo.name == index).order_by(
                desc(models.StreamerInfo.id)).first()
            if dbinfo:
                for dbfile in dbinfo.filelist:
                    save.append(dbfile.file)
        for file_name in os.listdir('.'):
            # 可能有两层后缀.with_suffix('')去掉一层.stem取文件名
            if (index in file_name or pathlib.Path(file_name).with_suffix('').stem in save ) and os.path.isfile(file_name):
                file_list.append(file_name)
        if len(file_list) == 0:
            return []

        file_list = sorted(file_list, key=lambda x: get_file_create_timestamp(x))

        # 正在上传的文件列表
        upload_filename: list = event_manager.context['upload_filename']

        results = []
        for index, file in enumerate(file_list):
            old_name = file
            if file.endswith('.part'):
                file_list[index] = os.path.splitext(file)[0]
                file = os.path.splitext(file)[0]

            name, ext = os.path.splitext(file)
            # 过滤正在上传的
            if name in upload_filename:
                continue
            # 过滤不是视频的
            if ext not in media_extensions:
                continue

            if old_name != file:
                logger.info(f'{old_name} 已更名为 {file}')
                shutil.move(old_name, file)

            file_size = os.path.getsize(file) / 1024 / 1024
            threshold = config.get('filtering_threshold', 0)
            if file_size <= threshold:
                os.remove(file)
                logger.info(f'过滤删除 - {file}')
                continue

            video = file
            danmaku = None
            if f'{name}.xml' in file_list:
                danmaku = f'{name}.xml'

            result = UploadBase.FileInfo(video=video, danmaku=danmaku)
            results.append(result)

        # 过滤弹幕
        for file in file_list:
            name, ext = os.path.splitext(file)
            # 过滤正在上传的
            if name in upload_filename:
                continue
            if ext == '.xml':
                have_video = False
                for result in results:
                    if result.danmaku == file:
                        have_video = True
                        break
                if not have_video:
                    logger.info(f'无视频，过滤删除 - {file}')
                    UploadBase.remove_file(file)
        return results

    @staticmethod
    def remove_filelist(file_list: List[FileInfo]):
        for f in file_list:
            UploadBase.remove_file(f.video)
            if f.danmaku is not None:
                UploadBase.remove_file(f.danmaku)

    @staticmethod
    def remove_file(file: str):
        try:
            os.remove(file)
            logger.info(f'删除 - {file}')
        except:
            logger.warning(f'删除失败 - {file}')

    def upload(self, file_list: List[FileInfo]) -> List[FileInfo]:
        raise NotImplementedError()

    def start(self):
        from biliup.handler import event_manager
        # 保证一个name同时只有一个上传线程扫描文件列表
        lock = NamedLock(f'upload_file_list_{self.principal}')
        upload_filename_list = []
        try:
            lock.acquire()
            file_list = UploadBase.file_list(self.principal)

            if len(file_list) > 0:
                upload_filename_list = [os.path.splitext(file.video)[0] for file in file_list]

                logger.info('准备上传' + self.data["format_title"])
                with NamedLock('upload_filename'):
                    event_manager.context['upload_filename'].extend(upload_filename_list)
                lock.release()
                file_list = self.upload(file_list)
                return file_list
        finally:
            with NamedLock('upload_filename'):
                event_manager.context['upload_filename'] = list(
                    set(event_manager.context['upload_filename']) - set(upload_filename_list))
            if lock.locked():
                lock.release()



import time

import biliup.common.util
from . import logger
from ..common import tools
from ..engine.decorators import Plugin
from ..engine.download import DownloadBase


@Plugin.download(regexp=r'(?:https?://)?(?:(?:www)\.)yy\.com')
class YY(DownloadBase):
    def __init__(self, fname, url, suffix='flv'):
        super().__init__(fname, url, suffix)

    async def acheck_stream(self, is_check=False):
        self.fake_headers['content-type'] = 'text/plain;charset=UTF-8'
        self.fake_headers['referer'] = 'https://www.yy.com/'

        if len(self.url.split("www.yy.com/")) < 2:
            logger.debug("直播间地址错误")
            return False
        else:
            rid = self.url.split("www.yy.com/")[1].split('/')[0]

        try:
            millis_13 = int(round(time.time() * 1000))
            millis_10 = int(time.time())
            data = '{"head":{"seq":'+str(millis_13)+',"appidstr":"0","bidstr":"121","cidstr":"'+str(rid)+'","sidstr":"'+str(rid)+'","uid64":0,"client_type":108,"client_ver":"5.11.0-alpha.4","stream_sys_ver":1,"app":"yylive_web","playersdk_ver":"5.11.0-alpha.4","thundersdk_ver":"0","streamsdk_ver":"5.11.0-alpha.4"},"client_attribute":{"client":"web","model":"","cpu":"","graphics_card":"","os":"chrome","osversion":"106.0.0.0","vsdk_version":"","app_identify":"","app_version":"","business":"","width":"1536","height":"864","scale":"","client_type":8,"h265":0},"avp_parameter":{"version":1,"client_type":8,"service_type":0,"imsi":0,"send_time":'+str(millis_10)+',"line_seq":-1,"gear":4,"ssl":1,"stream_format":0}}'
            url = f"https://stream-manager.yy.com/v3/channel/streams?uid=0&cid={rid}&sid={rid}&appid=0&sequence={millis_13}&encode=json"

            result = (await biliup.common.util.client.post(url, timeout=30, headers=self.fake_headers, data=data)).json()
            if 'avp_info_res' in result:
                a = result['avp_info_res']['stream_line_addr']
                self.raw_stream_url = list(a.values())[0]['cdn_info']['url']
                return True
            else:
                logger.debug(f'主播{rid}未开播')
                return False

        except Exception as e:
            logger.error(f"获取直播间信息失败{rid}")
            logger.error(e)
            return False



# Tencent is pleased to support the open source community by making Tars available.
#
# Copyright (C) 2016THL A29 Limited, a Tencent company. All rights reserved.
#
# Licensed under the BSD 3-Clause License (the "License"); you may not use this file except 
# in compliance with the License. You may obtain a copy of the License at
#
# https://opensource.org/licenses/BSD-3-Clause
#
# Unless required by applicable law or agreed to in writing, software distributed 
# under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
# CONDITIONS OF ANY KIND, either express or implied. See the License for the 
# specific language governing permissions and limitations under the License.
#

class TarsException(Exception): pass

class TarsTarsDecodeRequireNotExist(TarsException):    pass
class TarsTarsDecodeMismatch(TarsException):           pass
class TarsTarsDecodeInvalidValue(TarsException):       pass
class TarsTarsUnsupportType(TarsException):            pass

class TarsNetConnectException(TarsException):         pass
class TarsNetConnectLostException(TarsException):     pass
class TarsNetSocketException(TarsException):          pass
class TarsProxyDecodeException(TarsException):        pass
class TarsProxyEncodeException(TarsException):        pass
class TarsServerEncodeException(TarsException):       pass
class TarsServerDecodeException(TarsException):       pass
class TarsServerNoFuncException(TarsException):       pass
class TarsServerNoServantException(TarsException):    pass
class TarsServerQueueTimeoutException(TarsException): pass
class TarsServerUnknownException(TarsException):      pass
class TarsSyncCallTimeoutException(TarsException):    pass
class TarsRegistryException(TarsException):           pass
class TarsServerResetGridException(TarsException):    pass



# Tencent is pleased to support the open source community by making Tars available.
#
# Copyright (C) 2016THL A29 Limited, a Tencent company. All rights reserved.
#
# Licensed under the BSD 3-Clause License (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# https://opensource.org/licenses/BSD-3-Clause
#
# Unless required by applicable law or agreed to in writing, software distributed
# under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied. See the License for the
# specific language governing permissions and limitations under the License.
#

from biliup.Danmaku.tars.core import tarscore


class EndpointF(tarscore.struct):
    __tars_class__ = "register.EndpointF"

    def __init__(self):
        self.host = ""
        self.port = 0
        self.timeout = 0
        self.istcp = 0
        self.grid = 0
        self.groupworkid = 0
        self.grouprealid = 0
        self.setId = ""
        self.qos = 0
        self.bakFlag = 0
        self.weight = 0
        self.weightType = 0

    @staticmethod
    def writeTo(oos, value):
        oos.write(tarscore.string, 0, value.host)
        oos.write(tarscore.int32, 1, value.port)
        oos.write(tarscore.int32, 2, value.timeout)
        oos.write(tarscore.int32, 3, value.istcp)
        oos.write(tarscore.int32, 4, value.grid)
        oos.write(tarscore.int32, 5, value.groupworkid)
        oos.write(tarscore.int32, 6, value.grouprealid)
        oos.write(tarscore.string, 7, value.setId)
        oos.write(tarscore.int32, 8, value.qos)
        oos.write(tarscore.int32, 9, value.bakFlag)
        oos.write(tarscore.int32, 11, value.weight)
        oos.write(tarscore.int32, 12, value.weightType)

    @staticmethod
    def readFrom(ios):
        value = EndpointF()
        value.host = ios.read(tarscore.string, 0, True, value.host)
        value.port = ios.read(tarscore.int32, 1, True, value.port)
        value.timeout = ios.read(tarscore.int32, 2, True, value.timeout)
        value.istcp = ios.read(tarscore.int32, 3, True, value.istcp)
        value.grid = ios.read(tarscore.int32, 4, True, value.grid)
        value.groupworkid = ios.read(
            tarscore.int32, 5, False, value.groupworkid)
        value.grouprealid = ios.read(
            tarscore.int32, 6, False, value.grouprealid)
        value.setId = ios.read(tarscore.string, 7, False, value.setId)
        value.qos = ios.read(tarscore.int32, 8, False, value.qos)
        value.bakFlag = ios.read(tarscore.int32, 9, False, value.bakFlag)
        value.weight = ios.read(tarscore.int32, 11, False, value.weight)
        value.weightType = ios.read(
            tarscore.int32, 12, False, value.weightType)
        return value



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# filename: __rpc.py

# Tencent is pleased to support the open source community by making Tars available.
#
# Copyright (C) 2016THL A29 Limited, a Tencent company. All rights reserved.
#
# Licensed under the BSD 3-Clause License (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# https://opensource.org/licenses/BSD-3-Clause
#
# Unless required by applicable law or agreed to in writing, software distributed
# under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied. See the License for the
# specific language governing permissions and limitations under the License.
#

'''
@version: 0.01
@brief: 异步rpc实现
'''

import queue
import threading

from biliup.Danmaku.tars.__logger import tarsLogger
from biliup.Danmaku.tars.__packet import ResponsePacket
from biliup.Danmaku.tars.__servantproxy import ServantProxy


class AsyncProcThread:
    '''
    @brief: 异步调用线程管理类
    '''

    def __init__(self):
        tarsLogger.debug('AsyncProcThread:__init__')
        self.__initialize = False
        self.__runners = []
        self.__queue = None
        self.__nrunner = 0
        self.__popTimeout = 0.1

    def __del__(self):
        tarsLogger.debug('AsyncProcThread:__del__')

    def initialize(self, nrunner=3):
        '''
        @brief: 使用AsyncProcThread前必须先调用此函数
        @param nrunner: 异步线程个数
        @type nrunner: int
        @return: None
        @rtype: None
        '''
        tarsLogger.debug('AsyncProcThread:initialize')
        if self.__initialize:
            return
        self.__nrunner = nrunner
        self.__queue = queue.Queue()
        self.__initialize = True

    def terminate(self):
        '''
        @brief: 关闭所有异步线程
        @return: None
        @rtype: None
        '''
        tarsLogger.debug('AsyncProcThread:terminate')

        for runner in self.__runners:
            runner.terminate()

        for runner in self.__runners:
            runner.join()
        self.__runners = []

    def put(self, reqmsg):
        '''
        @brief: 处理数据入队列
        @param reqmsg: 待处理数据
        @type reqmsg: ReqMessage
        @return: None
        @rtype: None
        '''
        tarsLogger.debug('AsyncProcThread:put')
        # 异步请求超时
        if not reqmsg.response:
            reqmsg.response = ResponsePacket()
            reqmsg.response.iVerson = reqmsg.request.iVerson
            reqmsg.response.cPacketType = reqmsg.request.cPacketType
            reqmsg.response.iRequestId = reqmsg.request.iRequestId
            reqmsg.response.iRet = ServantProxy.TARSASYNCCALLTIMEOUT

        self.__queue.put(reqmsg)

    def pop(self):
        '''
        @brief: 处理数据出队列
        @return: ReqMessage
        @rtype: ReqMessage
        '''
        # tarsLogger.debug('AsyncProcThread:pop')
        ret = None
        try:
            ret = self.__queue.get(True, self.__popTimeout)
        except queue.Empty:
            pass
        return ret

    def start(self):
        '''
        @brief: 启动异步线程
        @return: None
        @rtype: None
        '''
        tarsLogger.debug('AsyncProcThread:start')
        for i in range(self.__nrunner):
            runner = AsyncProcThreadRunner()
            runner.initialize(self)
            runner.start()
            self.__runners.append(runner)


class AsyncProcThreadRunner(threading.Thread):
    '''
    @brief: 异步调用线程
    '''

    def __init__(self):
        tarsLogger.debug('AsyncProcThreadRunner:__init__')
        super(AsyncProcThreadRunner, self).__init__()
        # threading.Thread.__init__(self)
        self.__terminate = False
        self.__initialize = False
        self.__procQueue = None

    def __del__(self):
        tarsLogger.debug('AsyncProcThreadRunner:__del__')

    def initialize(self, queue):
        '''
        @brief: 使用AsyncProcThreadRunner前必须调用此函数
        @param queue: 有pop()的类，用于提取待处理数据
        @type queue: AsyncProcThread
        @return: None
        @rtype: None
        '''
        tarsLogger.debug('AsyncProcThreadRunner:initialize')
        self.__procQueue = queue

    def terminate(self):
        '''
        @brief: 关闭线程
        @return: None
        @rtype: None
        '''
        tarsLogger.debug('AsyncProcThreadRunner:terminate')
        self.__terminate = True

    def run(self):
        '''
        @brief: 线程启动函数，执行异步调用
        '''
        tarsLogger.debug('AsyncProcThreadRunner:run')
        while not self.__terminate:
            if self.__terminate:
                break
            reqmsg = self.__procQueue.pop()
            if not reqmsg or not reqmsg.callback:
                continue

            if reqmsg.adapter:
                succ = reqmsg.response.iRet == ServantProxy.TARSSERVERSUCCESS
                reqmsg.adapter.finishInvoke(succ)

            try:
                reqmsg.callback.onDispatch(reqmsg)
            except Exception as msg:
                tarsLogger.error('AsyncProcThread excepttion: %s', msg)

        tarsLogger.debug('AsyncProcThreadRunner:run finished')


class ServantProxyCallback(object):
    '''
    @brief: 异步回调对象基类
    '''

    def __init__(self):
        tarsLogger.debug('ServantProxyCallback:__init__')

    def onDispatch(reqmsg):
        '''
        @brief: 分配响应报文到对应的回调函数
        @param queue: 有pop()的类，用于提取待处理数据
        @type queue: AsyncProcThread
        @return: None
        @rtype: None
        '''
        raise NotImplementedError()



import time

import stream_gears
from typing import List

from ..engine import Plugin
from ..engine.upload import UploadBase, logger


@Plugin.upload(platform="stream_gears")
class BiliWeb(UploadBase):
    def __init__(
            self, principal, data, submit_api=None, copyright=2, postprocessor=None, dtime=None,
            dynamic='', lines='AUTO', threads=3, tid=122, tags=None, cover_path=None, description='',
            dolby=0, hires=0, no_reprint=0, open_elec=0, credits=None,
            user_cookie='cookies.json', copyright_source=None
    ):
        super().__init__(principal, data, persistence_path='bili.cookie', postprocessor=postprocessor)
        if credits is None:
            credits = []
        if tags is None:
            tags = []
        self.lines = lines
        self.submit_api = submit_api
        self.threads = threads
        self.tid = tid
        self.tags = tags
        if cover_path:
            self.cover_path = cover_path
        elif "live_cover_path" in self.data:
            self.cover_path = self.data["live_cover_path"]
        else:
            self.cover_path = None
        self.desc = description
        self.credits = credits
        self.dynamic = dynamic
        self.copyright = copyright
        self.dtime = dtime
        self.dolby = dolby
        self.hires = hires
        self.no_reprint = no_reprint
        self.open_elec = open_elec
        self.user_cookie = user_cookie
        self.copyright_source = copyright_source

    def upload(self, file_list) -> List[UploadBase.FileInfo]:
        line = None
        if self.lines == 'kodo':
            line = stream_gears.UploadLine.Kodo
        elif self.lines == 'bda2':
            line = stream_gears.UploadLine.Bda2
        elif self.lines == 'ws':
            line = stream_gears.UploadLine.Ws
        elif self.lines == 'qn':
            line = stream_gears.UploadLine.Qn
        elif self.lines == 'cos':
            line = stream_gears.UploadLine.Cos
        elif self.lines == 'cos-internal':
            line = stream_gears.UploadLine.CosInternal
        tag = ','.join(self.tags)
        if self.credits:
            desc_v2 = self.creditsToDesc_v2()
        else:
            desc_v2 = [{
                "raw_text": self.desc,
                "biz_id": "",
                "type": 1
            }]

        source = self.copyright_source if self.copyright_source else "https://github.com/biliup/biliup"
        cover = self.cover_path if self.cover_path is not None else ""
        dtime = None
        if self.dtime:
            dtime = int(time.time() + self.dtime)
        stream_gears.upload(
            file_list,
            self.user_cookie,
            self.data["format_title"][:80],
            self.tid,
            tag,
            self.copyright,
            source,
            self.desc,
            self.dynamic,
            cover,
            self.dolby,
            self.hires,
            self.no_reprint,
            self.open_elec,
            self.threads,
            desc_v2,
            dtime,
            line
        )
        logger.info(f"上传成功: {self.principal}")
        return file_list

    def creditsToDesc_v2(self):
        desc_v2 = []
        desc_v2_tmp = self.desc
        for credit in self.credits:
            try:
                num = desc_v2_tmp.index("@credit")
                desc_v2.append({
                    "raw_text": " " + desc_v2_tmp[:num],
                    "biz_id": "",
                    "type": 1
                })
                desc_v2.append({
                    "raw_text": credit["username"],
                    "biz_id": str(credit["uid"]),
                    "type": 2
                })
                self.desc = self.desc.replace(
                    "@credit", "@" + credit["username"] + "  ", 1)
                desc_v2_tmp = desc_v2_tmp[num + 7:]
            except IndexError:
                logger.error('简介中的@credit占位符少于credits的数量,替换失败')
        desc_v2.append({
            "raw_text": " " + desc_v2_tmp,
            "biz_id": "",
            "type": 1
        })
        desc_v2[0]["raw_text"] = desc_v2[0]["raw_text"][1:]  # 开头空格会导致识别简介过长
        return desc_v2



import biliup.common.util
from . import match1, logger
from ..common import tools
# from biliup.config import config
from ..engine.decorators import Plugin
from ..engine.download import DownloadBase

@Plugin.download(regexp=r'(?:https?://)?(?:(?:www|fm)\.)?missevan\.com')
class Missevan(DownloadBase):
    def __init__(self, fname, url, suffix='flv'):
        super().__init__(fname, url, suffix)

    async def acheck_stream(self, is_check=False):
        rid = 0
        # 用户主页获取直播间地址
        if self.url.split('www'):
            user_page = await biliup.common.util.client.get(self.url, timeout=30, headers=self.fake_headers)
            # 取硬编码在网页内的直播间号
            if user_page.status_code == 200:
                start = user_page.text.find('data-id="') + 9
                end = user_page.text.find('"', start)
                rid = user_page.text[start:end]
            else:
                logger.debug(user_page.status_code)
        if self.url.split("live"):
            rid = match1(self.url, r'/(\d+)')

        room_info = (await biliup.common.util.client.get(f"https://fm.missevan.com/api/v2/live/{rid}", timeout=30, headers=self.fake_headers)).json()

        # 无直播间的情况
        if room_info['code'] != 0:
            logger.debug(room_info['info'])
            return False

        # 开播状态
        if room_info['info']['room']['status']['open'] == 0:
            creator_username = room_info['info']['room']['creator_username']
            logger.debug(f"主播{creator_username}未开播")
            return False

        self.room_title = room_info['info']['room']['name']
        # if (config.get('missevanChannel') == 'flv'):
        #     self.raw_stream_url = room_info['info']['room']['channel']['flv_pull_url']
        # else:
        #     self.raw_stream_url = room_info['info']['room']['channel']['hls_pull_url']
        self.raw_stream_url = room_info['info']['room']['channel']['flv_pull_url']
        return True



from . import enc
from base64 import urlsafe_b64encode as b64enc
from urllib.parse import quote


def _header(video_id, channel_id) -> str:
    S1_3 = enc.rs(1, video_id)
    S1_5 = enc.rs(1, channel_id) + enc.rs(2, video_id)
    S1 = enc.rs(3, S1_3) + enc.rs(5, S1_5)
    S3 = enc.rs(48687757, enc.rs(1, video_id))
    header_replay = enc.rs(1, S1) + enc.rs(3, S3) + enc.nm(4, 1)
    return b64enc(header_replay)


def _build(video_id, seektime, topchat_only, channel_id) -> str:
    chattype = 4 if topchat_only else 1
    if seektime < 0:
        seektime = 0
    timestamp = int(seektime * 1000000)
    header = enc.rs(3, _header(video_id, channel_id))
    timestamp = enc.nm(5, timestamp)
    s6 = enc.nm(6, 0)
    s7 = enc.nm(7, 0)
    s8 = enc.nm(8, 0)
    s9 = enc.nm(9, 4)
    s10 = enc.rs(10, enc.nm(4, 0))
    chattype = enc.rs(14, enc.nm(1, 4))
    s15 = enc.nm(15, 0)
    entity = b"".join((header, timestamp, s6, s7, s8, s9, s10, chattype, s15))
    continuation = enc.rs(156074452, entity)
    return quote(b64enc(continuation).decode())


def getparam(video_id, seektime=0, topchat_only=False, channel_id="") -> str:
    """
    Parameter
    ---------
    seektime : int
        unit:seconds
        start position of fetching chat data.
    topchat_only : bool
        if True, fetch only 'top chat'
    """
    return _build(video_id, seektime, topchat_only, channel_id)



import biliup.common.util
from ..common import tools
from ..engine.decorators import Plugin
from ..engine.download import DownloadBase
from ..plugins import logger, match1


@Plugin.download(regexp=r'(?:https?://)?www\.flextv\.co\.kr')
class FlexTvCoKr(DownloadBase):
    def __init__(self, fname, url, suffix='flv'):
        super().__init__(fname, url, suffix)

    async def acheck_stream(self, is_check=False):
        room_id = match1(self.url, r"/channels/(\d+)/live")
        if not room_id:
            logger.warning(f"{FlexTvCoKr.__name__}: {self.url}: 直播间地址错误")
        response = await biliup.common.util.client.get(f"https://api.flextv.co.kr/api/channels/{room_id}/stream?option=all",
                                                       timeout=5,
                                                       headers=self.fake_headers)
        if response.status_code != 200:
            if response.status_code == 400:
                logger.debug(f"{FlexTvCoKr.__name__}: {self.url}: 未开播或直播间不存在")
                return False
            else:
                logger.warning(f"{FlexTvCoKr.__name__}: {self.url}: 获取错误，本次跳过")
                return False

        room_info = response.json()
        self.room_title = room_info['title']
        self.live_cover_url = room_info['thumbUrl']
        if is_check:
            return True

        m3u8_content = (await biliup.common.util.client.get(room_info['sources'][0]['url'], timeout=5, headers=self.fake_headers)).text
        import m3u8
        m3u8_obj = m3u8.loads(m3u8_content)
        if m3u8_obj.is_variant:
            # 取码率最大的流
            max_ratio_stream = max(m3u8_obj.playlists, key=lambda x: x.stream_info.bandwidth)
            self.raw_stream_url = max_ratio_stream.uri
        else:
            logger.warning(f"{FlexTvCoKr.__name__}: {self.url}: 解析错误")
            return False

        return True



import multiprocessing as mp
import time
from typing import List

import stream_gears

from ..engine import Plugin
from ..engine.upload import UploadBase, logger


@Plugin.upload(platform="biliup-rs")
class BiliWeb(UploadBase):
    def __init__(
            self, principal, data, submit_api=None, copyright=2, postprocessor=None, dtime=None,
            dynamic='', lines='AUTO', threads=3, tid=122, tags=None, cover_path=None, description='',
            dolby=0, hires=0, no_reprint=0, open_elec=0, credits=None,
            user_cookie='cookies.json', copyright_source=None
    ):
        super().__init__(principal, data, persistence_path='bili.cookie', postprocessor=postprocessor)
        if tags is None:
            tags = []
        else:
            tags = [str(tag).format(streamer=self.data['name']) for tag in tags]
        self.lines = lines
        self.submit_api = submit_api
        self.threads = threads
        self.tid = tid
        self.tags = tags
        if cover_path:
            self.cover_path = cover_path
        elif "live_cover_path" in self.data:
            self.cover_path = self.data["live_cover_path"]
        else:
            self.cover_path = None
        self.desc = description
        self.credits = credits if credits else []
        self.dynamic = dynamic
        self.copyright = copyright
        self.dtime = dtime
        self.dolby = dolby
        self.hires = hires
        self.no_reprint = no_reprint
        self.open_elec = open_elec
        self.user_cookie = user_cookie
        self.copyright_source = copyright_source

    def upload(self, file_list: List[UploadBase.FileInfo]) -> List[UploadBase.FileInfo]:
        if self.credits:
            desc_v2 = self.creditsToDesc_v2()
        else:
            desc_v2 = [{
                "raw_text": self.desc,
                "biz_id": "",
                "type": 1
            }]

        ex_parent_conn, ex_child_conn = mp.Pipe()
        upload_args = {
            "ex_conn": ex_child_conn,
            "lines": self.lines,
            "video_path": [file.video for file in file_list],
            "cookie_file": self.user_cookie,
            "title": self.data["format_title"][:80],
            "tid": self.tid,
            "tag": ','.join(self.tags),
            "copyright": self.copyright,
            "source": self.copyright_source if self.copyright_source else self.data["url"],
            "desc": self.desc,
            "dynamic": self.dynamic,
            "cover": self.cover_path if self.cover_path is not None else "",
            "dolby": self.dolby,
            "lossless_music": self.hires,
            "no_reprint": self.no_reprint,
            "open_elec": self.open_elec,
            "limit": self.threads,
            "desc_v2": desc_v2,
            "dtime": int(time.time() + self.dtime) if self.dtime else None,
        }

        upload_process = mp.get_context('spawn').Process(target=stream_gears_upload, daemon=True, kwargs=upload_args)
        upload_process.start()
        upload_process.join()
        if ex_parent_conn.poll():
            raise RuntimeError(ex_parent_conn.recv())

        logger.info(f"上传成功: {self.principal}")
        return file_list

    def creditsToDesc_v2(self):
        desc_v2 = []
        desc_v2_tmp = self.desc
        for credit in self.credits:
            try:
                num = desc_v2_tmp.index("@credit")
                desc_v2.append({
                    "raw_text": " " + desc_v2_tmp[:num],
                    "biz_id": "",
                    "type": 1
                })
                desc_v2.append({
                    "raw_text": credit["username"],
                    "biz_id": str(credit["uid"]),
                    "type": 2
                })
                self.desc = self.desc.replace(
                    "@credit", "@" + credit["username"] + "  ", 1)
                desc_v2_tmp = desc_v2_tmp[num + 7:]
            except IndexError:
                logger.error('简介中的@credit占位符少于credits的数量,替换失败')
        desc_v2.append({
            "raw_text": " " + desc_v2_tmp,
            "biz_id": "",
            "type": 1
        })
        desc_v2[0]["raw_text"] = desc_v2[0]["raw_text"][1:]  # 开头空格会导致识别简介过长
        return desc_v2


def stream_gears_upload(ex_conn, lines, *args, **kwargs):
    try:
        if lines == 'bda':
            kwargs['line'] = stream_gears.UploadLine.Bda
        elif lines == 'bda2':
            kwargs['line'] = stream_gears.UploadLine.Bda2
        elif lines == 'ws':
            kwargs['line'] = stream_gears.UploadLine.Ws
        elif lines == 'qn':
            kwargs['line'] = stream_gears.UploadLine.Qn
        elif lines == 'tx':
            kwargs['line'] = stream_gears.UploadLine.Tx
        elif lines == 'txa':
            kwargs['line'] = stream_gears.UploadLine.Txa
        elif lines == 'bldsa':
            kwargs['line'] = stream_gears.UploadLine.Bldsa

        stream_gears.upload(*args, **kwargs)
    except Exception as e:
        ex_conn.send(e)



import time
import random

import biliup.common.util
from biliup.config import config
from ..common import tools
from ..engine.decorators import Plugin
from ..engine.download import DownloadBase
from ..plugins import logger


@Plugin.download(regexp=r'(?:https?://)?(?:(?:live|www|v)\.)?(kuaishou)\.com')
@Plugin.download(regexp=r'(?:https?://)?(?:(?:(?:livev)\.(?:m))\.)?chenzhongtech\.com')
class Kuaishou(DownloadBase):
    def __init__(self, fname, url, suffix='flv'):
        super().__init__(fname, url, suffix)
        self.fake_headers['Cookie'] = config.get('kuaishou_cookie', '')

    async def acheck_stream(self, is_check=False):
        try:
            room_id = get_kwaiId(self.url)
            if not room_id:
                logger.warning(f"Kuaishou - {self.url}: 直播间地址错误")
                return False
        except Exception as e:
            logger.error(f"Kuaishou - {self.url}: {e}")
            return False

        plugin_msg = f"Kuaishou - {room_id}"

        # with requests.Session() as s:
        biliup.common.util.client.headers = self.fake_headers.copy()
        # 首页低风控生成did
        await biliup.common.util.client.get("https://live.kuaishou.com", timeout=5)

        # 不暂停似乎容易风控
        times = 3 + random.random()
        logger.debug(f"{plugin_msg}: 暂停 {times} 秒")
        time.sleep(times)

        err_keys = ["错误代码22", "主播尚未开播"]
        html = (await biliup.common.util.client.get(f"https://live.kuaishou.com/u/{room_id}", timeout=5)).text
        for key in err_keys:
            if key in html:
                logger.debug(f"{plugin_msg}: {key}")
                return False

        room_info = (await biliup.common.util.client.get(
            f"https://live.kuaishou.com/live_api/liveroom/livedetail?principalId={room_id}",
            timeout=5)).json()['data']

        if room_info['result'] == 22:
            logger.error(f"{plugin_msg}: 直播间地址错误")
            return False
        if room_info['result'] == 671:
            logger.debug(f"{plugin_msg}: 直播间未开播或非直播")
            return False
        if room_info['result'] != 1:
            logger.error(f"{plugin_msg}: {room_info}")
            return False

        if is_check:
            return True

        try:
            self.room_title = room_info['liveStream']['caption']
        except KeyError:
            logger.warning(f"{plugin_msg}: 直播间标题获取失败，使用快手ID代替")
            self.room_title = room_id
        self.raw_stream_url = room_info['liveStream']['playUrls'][0]['adaptationSet']['representation'][-1]['url']

        return True


def get_kwaiId(url):
    split_args = ["/profile/", "/fw/live/", "/u/"]
    for key in split_args:
        if key in url:
            kwaiId = url.split(key)[1]
            return kwaiId



import logging
import subprocess

logger = logging.getLogger('biliup')


class NamedLock:
    """
    简单实现的命名锁
    """
    from _thread import LockType
    _lock_dict = {}

    def __new__(cls, name) -> LockType:
        import threading
        if name not in cls._lock_dict:
            cls._lock_dict[name] = threading.Lock()
        return cls._lock_dict[name]


def silence_event_loop_closed(func):
    from functools import wraps

    @wraps(func)
    def wrapper(self, *args, **kwargs):
        try:
            return func(self, *args, **kwargs)
        except RuntimeError as e:
            if str(e) != 'Event loop is closed':
                raise

    return wrapper


def get_file_create_timestamp(file: str) -> float:
    """
    跨平台获取文件创建时间
    如无法获取则返回修改时间
    """
    import os
    import sys
    import platform
    stat_result = os.stat(file)

    if hasattr(stat_result, "st_birthtime"):
        return stat_result.st_birthtime

    if platform.system() == 'Windows' and sys.version_info < (3, 12):
        return stat_result.st_ctime

    if platform.system() == 'Linux':
        try:
            import subprocess
            time = float(subprocess.check_output(["stat", "-c", "%W", file]).decode('utf8'))
            if time > 0:
                return time
        except:
            pass

    return stat_result.st_mtime


def processor(processors, data):
    for process in processors:
        if process.get('run'):
            try:
                process_output = subprocess.check_output(
                    process['run'], shell=True,
                    input=data,
                    stderr=subprocess.STDOUT, text=True)
                logger.info(process_output.rstrip())
            except subprocess.CalledProcessError as e:
                logger.exception(e.output)
                continue



import copy
import json
import logging
from datetime import datetime
from pathlib import Path
from typing import List, Dict, Any

from sqlalchemy import create_engine, ForeignKey, JSON, TEXT, MetaData
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import Mapped, mapped_column, relationship, DeclarativeBase

logger = logging.getLogger('biliup')


# FIXME: 不应该在 stop 和 --version 时创建文件夹
def get_path(*other):
    """获取数据文件绝对路径"""
    dir_path = Path.cwd().joinpath("data")
    # 若目录不存在则创建
    if not dir_path.is_dir():
        dir_path.mkdir(parents=True)
    return str(dir_path.joinpath(*other))


DB_PATH = get_path('data.sqlite3')
DB_URL = f"sqlite:///{DB_PATH}"  # 数据库 URL, 使用默认 DBAPI

engine = create_engine(
    DB_URL, connect_args={"check_same_thread": False}
    # echo=True,  # 显示执行的 SQL 记录, 仅调试用, 发布前请注释
)

convention = {
    "ix": 'ix_%(column_0_label)s',
    "uq": "uq_%(table_name)s_%(column_0_name)s",
    "ck": "ck_%(table_name)s_%(constraint_name)s",
    "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
    "pk": "pk_%(table_name)s"
}

# Base = declarative_base()
class BaseModel(DeclarativeBase):
    """ 数据库表模型基类 """
    def as_dict(self):
        """ 将实例转为字典类型 """
        temp = copy.deepcopy(self.__dict__)  # 深复制避免对原数据影响
        result = dict()
        for key, value in temp.items():  # 遍历删除不能被 json 序列化的键值对
            if isinstance(value, datetime):  # 特殊处理，保留 datetime
                result[key] = value
                continue
            try:
                json.dumps(value)
                result[key] = value
            except TypeError:
                continue
        return result

    @classmethod
    def filter_parameters(cls, data: Dict[str, Any]):
        """ 过滤不需要的参数 """
        result = dict()
        for k, v in data.items():
            if (k in cls.__table__.c.keys()) or (k == "id"):
                result[k] = v
        return result


BaseModel.metadata = MetaData(naming_convention=convention)  # 定义命名惯例，避免生成自动迁移脚本时约束命名报错
# BaseModel.metadata.reflect(bind=engine)  # 绑定反射会导致表重复定义


class StreamerInfo(BaseModel):
    """下载信息"""
    __tablename__ = "streamerinfo"

    id: Mapped[int] = mapped_column(primary_key=True)  # 自增主键
    name: Mapped[str] = mapped_column(nullable=False)  # streamer 名称
    url: Mapped[str] = mapped_column(nullable=False)  # 录制的 url
    title: Mapped[str] = mapped_column(nullable=False)  # 直播标题
    date: Mapped[datetime] = mapped_column(nullable=False)  # 开播时间
    live_cover_path: Mapped[str] = mapped_column(nullable=False)  # 封面存储路径
    filelist: Mapped[List["FileList"]] = relationship(back_populates="streamerinfo")


class FileList(BaseModel):
    """存储文件名列表, 通过外键和 StreamerInfo 表关联"""
    __tablename__ = "filelist"

    id: Mapped[int] = mapped_column(primary_key=True)  # 自增主键
    file: Mapped[str] = mapped_column(nullable=False)  # 文件名
    # 外键, 对应 StreamerInfo 中的下载信息, 且启用级联删除
    streamer_info_id = mapped_column(ForeignKey("streamerinfo.id", ondelete="CASCADE"), nullable=False)
    streamerinfo: Mapped[StreamerInfo] = relationship(back_populates="filelist")


class Configuration(BaseModel):
    """暂时将配置文件整体存入，后续可拆表拆字段"""
    __tablename__ = "configuration"

    id: Mapped[int] = mapped_column(primary_key=True)  # 自增主键
    key: Mapped[str] = mapped_column(nullable=False)  # 全局配置键
    value = mapped_column(TEXT(), nullable=False)  # 全局配置值


class UploadStreamers(BaseModel):
    """投稿模板"""
    __tablename__ = "uploadstreamers"

    id: Mapped[int] = mapped_column(primary_key=True)  # 自增主键
    template_name: Mapped[str] = mapped_column(nullable=False)  # 模板名称
    title: Mapped[str] = mapped_column(
        nullable=True)  # 自定义标题的时间格式, {title}代表当场直播间标题 {streamer}代表在本config里面设置的主播名称 {url}代表设置的该主播的第一条直播间链接
    tid: Mapped[int] = mapped_column(nullable=True)  # 投稿分区码,171为电子竞技分区
    copyright: Mapped[int] = mapped_column(nullable=True)  # 1为自制 2转载
    copyright_source: Mapped[str] = mapped_column(nullable=True)  # 转载来源
    cover_path: Mapped[str] = mapped_column(nullable=True)  # 封面路径
    # 支持strftime, {title}, {streamer}, {url}占位符。
    description = mapped_column(TEXT(), nullable=True)  # 视频简介
    dynamic: Mapped[str] = mapped_column(nullable=True)  # 粉丝动态
    dtime: Mapped[int] = mapped_column(nullable=True)  # 设置延时发布时间，距离提交大于2小时，格式为时间戳
    dolby: Mapped[int] = mapped_column(nullable=True)  # 是否开启杜比音效, 1为开启
    hires: Mapped[int] = mapped_column(nullable=True)  # 是否开启Hi-Res, 1为开启
    open_elec: Mapped[int] = mapped_column(nullable=True)  # 是否开启充电面板, 1为开启
    no_reprint: Mapped[int] = mapped_column(nullable=True)  # 自制声明, 1为未经允许禁止转载
    uploader: Mapped[str] = mapped_column(nullable=True)  # 覆盖全局默认上传插件，Noop为不上传，但会执行后处理
    user_cookie: Mapped[str] = mapped_column(nullable=True)  # 使用指定的账号上传
    tags = mapped_column(JSON(), nullable=False)  # JSONField()  # 标签
    credits = mapped_column(JSON(), nullable=True)  # JSONField(null=True)  # 简介@模板
    up_selection_reply: Mapped[int] = mapped_column(nullable=True)  # 精选评论
    up_close_reply: Mapped[int] = mapped_column(nullable=True)  # 关闭评论
    up_close_danmu: Mapped[int] = mapped_column(nullable=True)  # 精选评论
    livestreamers: Mapped[List["LiveStreamers"]] = relationship(back_populates="uploadstreamers")


class LiveStreamers(BaseModel):
    """每个直播间的配置"""
    __tablename__ = "livestreamers"

    id: Mapped[int] = mapped_column(primary_key=True)  # 自增主键
    url: Mapped[str] = mapped_column(nullable=False, unique=True)  # 直播间地址
    remark: Mapped[str] = mapped_column(nullable=False)  # 对应配置文件中 {streamer} 变量
    filename_prefix: Mapped[str] = mapped_column(nullable=True)  # filename_prefix 支持模板
    # 外键, 对应 UploadStreamers, 且启用级联删除
    upload_streamers_id = mapped_column(ForeignKey("uploadstreamers.id", ondelete="CASCADE"), nullable=True)
    uploadstreamers: Mapped[UploadStreamers] = relationship(back_populates="livestreamers")
    format: Mapped[str] = mapped_column(nullable=True)  # 视频格式
    preprocessor = mapped_column(JSON(), nullable=True)  # 开始下载直播时触发
    segment_processor = mapped_column(JSON(), nullable=True)  # 分段时触发
    downloaded_processor = mapped_column(JSON(), nullable=True)  # 准备上传直播时触发
    postprocessor = mapped_column(JSON(), nullable=True)  # 上传完成后触发
    opt_args = mapped_column(JSON(), nullable=True)  # ffmpeg参数



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dy.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x08\x64y.proto\x12\x0b\x64ouyin_util\"\xee\x02\n\x08Response\x12*\n\x0cmessagesList\x18\x01 \x03(\x0b\x32\x14.douyin_util.Message\x12\x0e\n\x06\x63ursor\x18\x02 \x01(\t\x12\x15\n\rfetchInterval\x18\x03 \x01(\x04\x12\x0b\n\x03now\x18\x04 \x01(\x04\x12\x13\n\x0binternalExt\x18\x05 \x01(\t\x12\x11\n\tfetchType\x18\x06 \x01(\r\x12;\n\x0brouteParams\x18\x07 \x03(\x0b\x32&.douyin_util.Response.RouteParamsEntry\x12\x19\n\x11heartbeatDuration\x18\x08 \x01(\x04\x12\x0f\n\x07needAck\x18\t \x01(\x08\x12\x12\n\npushServer\x18\n \x01(\t\x12\x12\n\nliveCursor\x18\x0b \x01(\t\x12\x15\n\rhistoryNoMore\x18\x0c \x01(\x08\x1a\x32\n\x10RouteParamsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x9a\x01\n\x07Message\x12\x0e\n\x06method\x18\x01 \x01(\t\x12\x0f\n\x07payload\x18\x02 \x01(\x0c\x12\r\n\x05msgId\x18\x03 \x01(\x03\x12\x0f\n\x07msgType\x18\x04 \x01(\x05\x12\x0e\n\x06offset\x18\x05 \x01(\x03\x12\x15\n\rneedWrdsStore\x18\x06 \x01(\x08\x12\x13\n\x0bwrdsVersion\x18\x07 \x01(\x03\x12\x12\n\nwrdsSubKey\x18\x08 \x01(\t\"\xf2\x04\n\x0b\x43hatMessage\x12#\n\x06\x63ommon\x18\x01 \x01(\x0b\x32\x13.douyin_util.Common\x12\x1f\n\x04user\x18\x02 \x01(\x0b\x32\x11.douyin_util.User\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\x17\n\x0fvisibleToSender\x18\x04 \x01(\x08\x12+\n\x0f\x62\x61\x63kgroundImage\x18\x05 \x01(\x0b\x32\x12.douyin_util.Image\x12\x1b\n\x13\x66ullScreenTextColor\x18\x06 \x01(\t\x12-\n\x11\x62\x61\x63kgroundImageV2\x18\x07 \x01(\x0b\x32\x12.douyin_util.Image\x12\x37\n\x10publicAreaCommon\x18\x08 \x01(\x0b\x32\x1d.douyin_util.PublicAreaCommon\x12%\n\tgiftImage\x18\t \x01(\x0b\x32\x12.douyin_util.Image\x12\x12\n\nagreeMsgId\x18\x0b \x01(\x04\x12\x15\n\rpriorityLevel\x18\x0c \x01(\r\x12=\n\x13landscapeAreaCommon\x18\r \x01(\x0b\x32 .douyin_util.LandscapeAreaCommon\x12\x11\n\teventTime\x18\x0f \x01(\x04\x12\x12\n\nsendReview\x18\x10 \x01(\x08\x12\x14\n\x0c\x66romIntercom\x18\x11 \x01(\x08\x12\x1c\n\x14intercomHideUserCard\x18\x12 \x01(\x08\x12\x0e\n\x06\x63hatBy\x18\x14 \x01(\t\x12\x1e\n\x16individualChatPriority\x18\x15 \x01(\r\x12%\n\nrtfContent\x18\x16 \x01(\x0b\x32\x11.douyin_util.Text\"\xa6\x01\n\x13LandscapeAreaCommon\x12\x10\n\x08showHead\x18\x01 \x01(\x08\x12\x14\n\x0cshowNickname\x18\x02 \x01(\x08\x12\x15\n\rshowFontColor\x18\x03 \x01(\x08\x12\x16\n\x0e\x63olorValueList\x18\x04 \x03(\t\x12\x38\n\x13\x63ommentTypeTagsList\x18\x05 \x03(\x0e\x32\x1b.douyin_util.CommentTypeTag\"\x96\x03\n\x12RoomUserSeqMessage\x12#\n\x06\x63ommon\x18\x01 \x01(\x0b\x32\x13.douyin_util.Common\x12=\n\tranksList\x18\x02 \x03(\x0b\x32*.douyin_util.RoomUserSeqMessageContributor\x12\r\n\x05total\x18\x03 \x01(\x03\x12\x0e\n\x06popStr\x18\x04 \x01(\t\x12=\n\tseatsList\x18\x05 \x03(\x0b\x32*.douyin_util.RoomUserSeqMessageContributor\x12\x12\n\npopularity\x18\x06 \x01(\x03\x12\x11\n\ttotalUser\x18\x07 \x01(\x03\x12\x14\n\x0ctotalUserStr\x18\x08 \x01(\t\x12\x10\n\x08totalStr\x18\t \x01(\t\x12\x1b\n\x13onlineUserForAnchor\x18\n \x01(\t\x12\x18\n\x10totalPvForAnchor\x18\x0b \x01(\t\x12\x17\n\x0fupRightStatsStr\x18\x0c \x01(\t\x12\x1f\n\x17upRightStatsStrComplete\x18\r \x01(\t\"h\n\x11\x43ommonTextMessage\x12#\n\x06\x63ommon\x18\x01 \x01(\x0b\x32\x13.douyin_util.Common\x12\x1f\n\x04user\x18\x02 \x01(\x0b\x32\x11.douyin_util.User\x12\r\n\x05scene\x18\x03 \x01(\t\"\x8e\x01\n\x16UpdateFanTicketMessage\x12#\n\x06\x63ommon\x18\x01 \x01(\x0b\x32\x13.douyin_util.Common\x12\x1e\n\x16roomFanTicketCountText\x18\x02 \x01(\t\x12\x1a\n\x12roomFanTicketCount\x18\x03 \x01(\x04\x12\x13\n\x0b\x66orceUpdate\x18\x04 \x01(\x08\"\xae\x01\n\x1dRoomUserSeqMessageContributor\x12\r\n\x05score\x18\x01 \x01(\x04\x12\x1f\n\x04user\x18\x02 \x01(\x0b\x32\x11.douyin_util.User\x12\x0c\n\x04rank\x18\x03 \x01(\x04\x12\r\n\x05\x64\x65lta\x18\x04 \x01(\x04\x12\x10\n\x08isHidden\x18\x05 \x01(\x08\x12\x18\n\x10scoreDescription\x18\x06 \x01(\t\x12\x14\n\x0c\x65xactlyScore\x18\x07 \x01(\t\"\xd9\x06\n\x0bGiftMessage\x12#\n\x06\x63ommon\x18\x01 \x01(\x0b\x32\x13.douyin_util.Common\x12\x0e\n\x06giftId\x18\x02 \x01(\x04\x12\x16\n\x0e\x66\x61nTicketCount\x18\x03 \x01(\x04\x12\x12\n\ngroupCount\x18\x04 \x01(\x04\x12\x13\n\x0brepeatCount\x18\x05 \x01(\x04\x12\x12\n\ncomboCount\x18\x06 \x01(\x04\x12\x1f\n\x04user\x18\x07 \x01(\x0b\x32\x11.douyin_util.User\x12!\n\x06toUser\x18\x08 \x01(\x0b\x32\x11.douyin_util.User\x12\x11\n\trepeatEnd\x18\t \x01(\r\x12+\n\ntextEffect\x18\n \x01(\x0b\x32\x17.douyin_util.TextEffect\x12\x0f\n\x07groupId\x18\x0b \x01(\x04\x12\x17\n\x0fincomeTaskgifts\x18\x0c \x01(\x04\x12\x1a\n\x12roomFanTicketCount\x18\r \x01(\x04\x12-\n\x08priority\x18\x0e \x01(\x0b\x32\x1b.douyin_util.GiftIMPriority\x12%\n\x04gift\x18\x0f \x01(\x0b\x32\x17.douyin_util.GiftStruct\x12\r\n\x05logId\x18\x10 \x01(\t\x12\x10\n\x08sendType\x18\x11 \x01(\x04\x12\x37\n\x10publicAreaCommon\x18\x12 \x01(\x0b\x32\x1d.douyin_util.PublicAreaCommon\x12*\n\x0ftrayDisplayText\x18\x13 \x01(\x0b\x32\x11.douyin_util.Text\x12\x1c\n\x14\x62\x61nnedDisplayEffects\x18\x14 \x01(\x04\x12\x16\n\x0e\x64isplayForSelf\x18\x19 \x01(\x08\x12\x18\n\x10interactGiftInfo\x18\x1a \x01(\t\x12\x13\n\x0b\x64iyItemInfo\x18\x1b \x01(\t\x12\x17\n\x0fminAssetSetList\x18\x1c \x03(\x04\x12\x12\n\ntotalCount\x18\x1d \x01(\x04\x12\x18\n\x10\x63lientGiftSource\x18\x1e \x01(\r\x12\x15\n\rtoUserIdsList\x18  \x03(\x04\x12\x10\n\x08sendTime\x18! \x01(\x04\x12\x1b\n\x13\x66orceDisplayEffects\x18\" \x01(\x04\x12\x0f\n\x07traceId\x18# \x01(\t\x12\x17\n\x0f\x65\x66\x66\x65\x63tDisplayTs\x18$ \x01(\x04\"\xb2\x03\n\nGiftStruct\x12!\n\x05image\x18\x01 \x01(\x0b\x32\x12.douyin_util.Image\x12\x10\n\x08\x64\x65scribe\x18\x02 \x01(\t\x12\x0e\n\x06notify\x18\x03 \x01(\x08\x12\x10\n\x08\x64uration\x18\x04 \x01(\x04\x12\n\n\x02id\x18\x05 \x01(\x04\x12\x12\n\nforLinkmic\x18\x07 \x01(\x08\x12\x0e\n\x06\x64oodle\x18\x08 \x01(\x08\x12\x13\n\x0b\x66orFansclub\x18\t \x01(\x08\x12\r\n\x05\x63ombo\x18\n \x01(\x08\x12\x0c\n\x04type\x18\x0b \x01(\r\x12\x14\n\x0c\x64iamondCount\x18\x0c \x01(\r\x12\x1a\n\x12isDisplayedOnPanel\x18\r \x01(\x08\x12\x17\n\x0fprimaryEffectId\x18\x0e \x01(\x04\x12)\n\rgiftLabelIcon\x18\x0f \x01(\x0b\x32\x12.douyin_util.Image\x12\x0c\n\x04name\x18\x10 \x01(\t\x12\x0e\n\x06region\x18\x11 \x01(\t\x12\x0e\n\x06manual\x18\x12 \x01(\t\x12\x11\n\tforCustom\x18\x13 \x01(\x08\x12 \n\x04icon\x18\x15 \x01(\x0b\x32\x12.douyin_util.Image\x12\x12\n\nactionType\x18\x16 \x01(\r\"U\n\x0eGiftIMPriority\x12\x16\n\x0equeueSizesList\x18\x01 \x03(\x04\x12\x19\n\x11selfQueuePriority\x18\x02 \x01(\x04\x12\x10\n\x08priority\x18\x03 \x01(\x04\"o\n\nTextEffect\x12/\n\x08portrait\x18\x01 \x01(\x0b\x32\x1d.douyin_util.TextEffectDetail\x12\x30\n\tlandscape\x18\x02 \x01(\x0b\x32\x1d.douyin_util.TextEffectDetail\"\xc0\x02\n\x10TextEffectDetail\x12\x1f\n\x04text\x18\x01 \x01(\x0b\x32\x11.douyin_util.Text\x12\x14\n\x0ctextFontSize\x18\x02 \x01(\r\x12&\n\nbackground\x18\x03 \x01(\x0b\x32\x12.douyin_util.Image\x12\r\n\x05start\x18\x04 \x01(\r\x12\x10\n\x08\x64uration\x18\x05 \x01(\r\x12\t\n\x01x\x18\x06 \x01(\r\x12\t\n\x01y\x18\x07 \x01(\r\x12\r\n\x05width\x18\x08 \x01(\r\x12\x0e\n\x06height\x18\t \x01(\r\x12\x10\n\x08shadowDx\x18\n \x01(\r\x12\x10\n\x08shadowDy\x18\x0b \x01(\r\x12\x14\n\x0cshadowRadius\x18\x0c \x01(\r\x12\x13\n\x0bshadowColor\x18\r \x01(\t\x12\x13\n\x0bstrokeColor\x18\x0e \x01(\t\x12\x13\n\x0bstrokeWidth\x18\x0f \x01(\r\"\x9c\x05\n\rMemberMessage\x12#\n\x06\x63ommon\x18\x01 \x01(\x0b\x32\x13.douyin_util.Common\x12\x1f\n\x04user\x18\x02 \x01(\x0b\x32\x11.douyin_util.User\x12\x13\n\x0bmemberCount\x18\x03 \x01(\x04\x12#\n\x08operator\x18\x04 \x01(\x0b\x32\x11.douyin_util.User\x12\x14\n\x0cisSetToAdmin\x18\x05 \x01(\x08\x12\x11\n\tisTopUser\x18\x06 \x01(\x08\x12\x11\n\trankScore\x18\x07 \x01(\x04\x12\x11\n\ttopUserNo\x18\x08 \x01(\x04\x12\x11\n\tenterType\x18\t \x01(\x04\x12\x0e\n\x06\x61\x63tion\x18\n \x01(\x04\x12\x19\n\x11\x61\x63tionDescription\x18\x0b \x01(\t\x12\x0e\n\x06userId\x18\x0c \x01(\x04\x12/\n\x0c\x65\x66\x66\x65\x63tConfig\x18\r \x01(\x0b\x32\x19.douyin_util.EffectConfig\x12\x0e\n\x06popStr\x18\x0e \x01(\t\x12\x34\n\x11\x65nterEffectConfig\x18\x0f \x01(\x0b\x32\x19.douyin_util.EffectConfig\x12+\n\x0f\x62\x61\x63kgroundImage\x18\x10 \x01(\x0b\x32\x12.douyin_util.Image\x12-\n\x11\x62\x61\x63kgroundImageV2\x18\x11 \x01(\x0b\x32\x12.douyin_util.Image\x12,\n\x11\x61nchorDisplayText\x18\x12 \x01(\x0b\x32\x11.douyin_util.Text\x12\x37\n\x10publicAreaCommon\x18\x13 \x01(\x0b\x32\x1d.douyin_util.PublicAreaCommon\x12\x18\n\x10userEnterTipType\x18\x14 \x01(\x04\x12\x1a\n\x12\x61nchorEnterTipType\x18\x15 \x01(\x04\"s\n\x10PublicAreaCommon\x12%\n\tuserLabel\x18\x01 \x01(\x0b\x32\x12.douyin_util.Image\x12\x19\n\x11userConsumeInRoom\x18\x02 \x01(\x04\x12\x1d\n\x15userSendGiftCntInRoom\x18\x03 \x01(\x04\"\xbe\x05\n\x0c\x45\x66\x66\x65\x63tConfig\x12\x0c\n\x04type\x18\x01 \x01(\x04\x12 \n\x04icon\x18\x02 \x01(\x0b\x32\x12.douyin_util.Image\x12\x11\n\tavatarPos\x18\x03 \x01(\x04\x12\x1f\n\x04text\x18\x04 \x01(\x0b\x32\x11.douyin_util.Text\x12$\n\x08textIcon\x18\x05 \x01(\x0b\x32\x12.douyin_util.Image\x12\x10\n\x08stayTime\x18\x06 \x01(\r\x12\x13\n\x0b\x61nimAssetId\x18\x07 \x01(\x04\x12!\n\x05\x62\x61\x64ge\x18\x08 \x01(\x0b\x32\x12.douyin_util.Image\x12\x1c\n\x14\x66lexSettingArrayList\x18\t \x03(\x04\x12+\n\x0ftextIconOverlay\x18\n \x01(\x0b\x32\x12.douyin_util.Image\x12)\n\ranimatedBadge\x18\x0b \x01(\x0b\x32\x12.douyin_util.Image\x12\x15\n\rhasSweepLight\x18\x0c \x01(\x08\x12 \n\x18textFlexSettingArrayList\x18\r \x03(\x04\x12\x19\n\x11\x63\x65nterAnimAssetId\x18\x0e \x01(\x04\x12(\n\x0c\x64ynamicImage\x18\x0f \x01(\x0b\x32\x12.douyin_util.Image\x12\x39\n\x08\x65xtraMap\x18\x10 \x03(\x0b\x32\'.douyin_util.EffectConfig.ExtraMapEntry\x12\x16\n\x0emp4AnimAssetId\x18\x11 \x01(\x04\x12\x10\n\x08priority\x18\x12 \x01(\x04\x12\x13\n\x0bmaxWaitTime\x18\x13 \x01(\x04\x12\x0f\n\x07\x64ressId\x18\x14 \x01(\t\x12\x11\n\talignment\x18\x15 \x01(\x04\x12\x17\n\x0f\x61lignmentOffset\x18\x16 \x01(\x04\x1a/\n\rExtraMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x86\x01\n\x04Text\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x15\n\rdefaultPatter\x18\x02 \x01(\t\x12.\n\rdefaultFormat\x18\x03 \x01(\x0b\x32\x17.douyin_util.TextFormat\x12*\n\npiecesList\x18\x04 \x03(\x0b\x32\x16.douyin_util.TextPiece\"\xd2\x02\n\tTextPiece\x12\x0c\n\x04type\x18\x01 \x01(\x08\x12\'\n\x06\x66ormat\x18\x02 \x01(\x0b\x32\x17.douyin_util.TextFormat\x12\x13\n\x0bstringValue\x18\x03 \x01(\t\x12-\n\tuserValue\x18\x04 \x01(\x0b\x32\x1a.douyin_util.TextPieceUser\x12-\n\tgiftValue\x18\x05 \x01(\x0b\x32\x1a.douyin_util.TextPieceGift\x12/\n\nheartValue\x18\x06 \x01(\x0b\x32\x1b.douyin_util.TextPieceHeart\x12\x39\n\x0fpatternRefValue\x18\x07 \x01(\x0b\x32 .douyin_util.TextPiecePatternRef\x12/\n\nimageValue\x18\x08 \x01(\x0b\x32\x1b.douyin_util.TextPieceImage\"H\n\x0eTextPieceImage\x12!\n\x05image\x18\x01 \x01(\x0b\x32\x12.douyin_util.Image\x12\x13\n\x0bscalingRate\x18\x02 \x01(\x02\":\n\x13TextPiecePatternRef\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x16\n\x0e\x64\x65\x66\x61ultPattern\x18\x02 \x01(\t\"\x1f\n\x0eTextPieceHeart\x12\r\n\x05\x63olor\x18\x01 \x01(\t\"I\n\rTextPieceGift\x12\x0e\n\x06giftId\x18\x01 \x01(\x04\x12(\n\x07nameRef\x18\x02 \x01(\x0b\x32\x17.douyin_util.PatternRef\"1\n\nPatternRef\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x16\n\x0e\x64\x65\x66\x61ultPattern\x18\x02 \x01(\t\"C\n\rTextPieceUser\x12\x1f\n\x04user\x18\x01 \x01(\x0b\x32\x11.douyin_util.User\x12\x11\n\twithColon\x18\x02 \x01(\x08\"\xa3\x01\n\nTextFormat\x12\r\n\x05\x63olor\x18\x01 \x01(\t\x12\x0c\n\x04\x62old\x18\x02 \x01(\x08\x12\x0e\n\x06italic\x18\x03 \x01(\x08\x12\x0e\n\x06weight\x18\x04 \x01(\r\x12\x13\n\x0bitalicAngle\x18\x05 \x01(\r\x12\x10\n\x08\x66ontSize\x18\x06 \x01(\r\x12\x1a\n\x12useHeighLightColor\x18\x07 \x01(\x08\x12\x15\n\ruseRemoteClor\x18\x08 \x01(\x08\"\xe3\x02\n\x0bLikeMessage\x12#\n\x06\x63ommon\x18\x01 \x01(\x0b\x32\x13.douyin_util.Common\x12\r\n\x05\x63ount\x18\x02 \x01(\x04\x12\r\n\x05total\x18\x03 \x01(\x04\x12\r\n\x05\x63olor\x18\x04 \x01(\x04\x12\x1f\n\x04user\x18\x05 \x01(\x0b\x32\x11.douyin_util.User\x12\x0c\n\x04icon\x18\x06 \x01(\t\x12\x37\n\x10\x64oubleLikeDetail\x18\x07 \x01(\x0b\x32\x1d.douyin_util.DoubleLikeDetail\x12;\n\x12\x64isplayControlInfo\x18\x08 \x01(\x0b\x32\x1f.douyin_util.DisplayControlInfo\x12\x17\n\x0flinkmicGuestUid\x18\t \x01(\x04\x12\r\n\x05scene\x18\n \x01(\t\x12\x35\n\x0fpicoDisplayInfo\x18\x0b \x01(\x0b\x32\x1c.douyin_util.PicoDisplayInfo\"\xdb\x01\n\rSocialMessage\x12#\n\x06\x63ommon\x18\x01 \x01(\x0b\x32\x13.douyin_util.Common\x12\x1f\n\x04user\x18\x02 \x01(\x0b\x32\x11.douyin_util.User\x12\x11\n\tshareType\x18\x03 \x01(\x04\x12\x0e\n\x06\x61\x63tion\x18\x04 \x01(\x04\x12\x13\n\x0bshareTarget\x18\x05 \x01(\t\x12\x13\n\x0b\x66ollowCount\x18\x06 \x01(\x04\x12\x37\n\x10publicAreaCommon\x18\x07 \x01(\x0b\x32\x1d.douyin_util.PublicAreaCommon\"q\n\x0fPicoDisplayInfo\x12\x15\n\rcomboSumCount\x18\x01 \x01(\x04\x12\r\n\x05\x65moji\x18\x02 \x01(\t\x12%\n\temojiIcon\x18\x03 \x01(\x0b\x32\x12.douyin_util.Image\x12\x11\n\temojiText\x18\x04 \x01(\t\"_\n\x10\x44oubleLikeDetail\x12\x12\n\ndoubleFlag\x18\x01 \x01(\x08\x12\r\n\x05seqId\x18\x02 \x01(\r\x12\x13\n\x0brenewalsNum\x18\x03 \x01(\r\x12\x13\n\x0btriggersNum\x18\x04 \x01(\r\"9\n\x12\x44isplayControlInfo\x12\x10\n\x08showText\x18\x01 \x01(\x08\x12\x11\n\tshowIcons\x18\x02 \x01(\x08\"\xd7\x01\n\x12\x45pisodeChatMessage\x12$\n\x06\x63ommon\x18\x01 \x01(\x0b\x32\x14.douyin_util.Message\x12\x1f\n\x04user\x18\x02 \x01(\x0b\x32\x11.douyin_util.User\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\x16\n\x0evisibleToSende\x18\x04 \x01(\x08\x12%\n\tgiftImage\x18\x07 \x01(\x0b\x32\x12.douyin_util.Image\x12\x12\n\nagreeMsgId\x18\x08 \x01(\x04\x12\x16\n\x0e\x63olorValueList\x18\t \x03(\t\"\x92\x01\n\x18MatchAgainstScoreMessage\x12#\n\x06\x63ommon\x18\x01 \x01(\x0b\x32\x13.douyin_util.Common\x12%\n\x07\x61gainst\x18\x02 \x01(\x0b\x32\x14.douyin_util.Against\x12\x13\n\x0bmatchStatus\x18\x03 \x01(\r\x12\x15\n\rdisplayStatus\x18\x04 \x01(\r\"\x9c\x03\n\x07\x41gainst\x12\x10\n\x08leftName\x18\x01 \x01(\t\x12$\n\x08leftLogo\x18\x02 \x01(\x0b\x32\x12.douyin_util.Image\x12\x10\n\x08leftGoal\x18\x03 \x01(\t\x12\x11\n\trightName\x18\x06 \x01(\t\x12%\n\trightLogo\x18\x07 \x01(\x0b\x32\x12.douyin_util.Image\x12\x11\n\trightGoal\x18\x08 \x01(\t\x12\x11\n\ttimestamp\x18\x0b \x01(\x04\x12\x0f\n\x07version\x18\x0c \x01(\x04\x12\x12\n\nleftTeamId\x18\r \x01(\x04\x12\x13\n\x0brightTeamId\x18\x0e \x01(\x04\x12\x19\n\x11\x64iffSei2absSecond\x18\x0f \x01(\x04\x12\x16\n\x0e\x66inalGoalStage\x18\x10 \x01(\r\x12\x18\n\x10\x63urrentGoalStage\x18\x11 \x01(\r\x12\x19\n\x11leftScoreAddition\x18\x12 \x01(\r\x12\x1a\n\x12rightScoreAddition\x18\x13 \x01(\r\x12\x13\n\x0bleftGoalInt\x18\x14 \x01(\x04\x12\x14\n\x0crightGoalInt\x18\x15 \x01(\x04\"\xd6\x03\n\x06\x43ommon\x12\x0e\n\x06method\x18\x01 \x01(\t\x12\r\n\x05msgId\x18\x02 \x01(\x04\x12\x0e\n\x06roomId\x18\x03 \x01(\x04\x12\x12\n\ncreateTime\x18\x04 \x01(\x04\x12\x0f\n\x07monitor\x18\x05 \x01(\r\x12\x11\n\tisShowMsg\x18\x06 \x01(\x08\x12\x10\n\x08\x64\x65scribe\x18\x07 \x01(\t\x12\x10\n\x08\x66oldType\x18\t \x01(\x04\x12\x16\n\x0e\x61nchorFoldType\x18\n \x01(\x04\x12\x15\n\rpriorityScore\x18\x0b \x01(\x04\x12\r\n\x05logId\x18\x0c \x01(\t\x12\x19\n\x11msgProcessFilterK\x18\r \x01(\t\x12\x19\n\x11msgProcessFilterV\x18\x0e \x01(\t\x12\x1f\n\x04user\x18\x0f \x01(\x0b\x32\x11.douyin_util.User\x12\x18\n\x10\x61nchorFoldTypeV2\x18\x11 \x01(\x04\x12\x1a\n\x12processAtSeiTimeMs\x18\x12 \x01(\x04\x12\x18\n\x10randomDispatchMs\x18\x13 \x01(\x04\x12\x12\n\nisDispatch\x18\x14 \x01(\x08\x12\x11\n\tchannelId\x18\x15 \x01(\x04\x12\x19\n\x11\x64iffSei2absSecond\x18\x16 \x01(\x04\x12\x1a\n\x12\x61nchorFoldDuration\x18\x17 \x01(\x04\"\xf3\x04\n\x04User\x12\n\n\x02id\x18\x01 \x01(\x04\x12\x0f\n\x07shortId\x18\x02 \x01(\x04\x12\x10\n\x08nickName\x18\x03 \x01(\t\x12\x0e\n\x06gender\x18\x04 \x01(\r\x12\x11\n\tSignature\x18\x05 \x01(\t\x12\r\n\x05Level\x18\x06 \x01(\r\x12\x10\n\x08\x42irthday\x18\x07 \x01(\x04\x12\x11\n\tTelephone\x18\x08 \x01(\t\x12\'\n\x0b\x41vatarThumb\x18\t \x01(\x0b\x32\x12.douyin_util.Image\x12(\n\x0c\x41vatarMedium\x18\n \x01(\x0b\x32\x12.douyin_util.Image\x12\'\n\x0b\x41vatarLarge\x18\x0b \x01(\x0b\x32\x12.douyin_util.Image\x12\x10\n\x08Verified\x18\x0c \x01(\x08\x12\x12\n\nExperience\x18\r \x01(\r\x12\x0c\n\x04\x63ity\x18\x0e \x01(\t\x12\x0e\n\x06Status\x18\x0f \x01(\x05\x12\x12\n\nCreateTime\x18\x10 \x01(\x04\x12\x12\n\nModifyTime\x18\x11 \x01(\x04\x12\x0e\n\x06Secret\x18\x12 \x01(\r\x12\x16\n\x0eShareQrcodeUri\x18\x13 \x01(\t\x12\x1a\n\x12IncomeSharePercent\x18\x14 \x01(\r\x12*\n\x0e\x42\x61\x64geImageList\x18\x15 \x03(\x0b\x32\x12.douyin_util.Image\x12\x11\n\tSpecialId\x18\x1a \x01(\t\x12(\n\x0c\x41vatarBorder\x18\x1b \x01(\x0b\x32\x12.douyin_util.Image\x12!\n\x05Medal\x18\x1c \x01(\x0b\x32\x12.douyin_util.Image\x12-\n\x11RealTimeIconsList\x18\x1d \x03(\x0b\x32\x12.douyin_util.Image\"\xb1\x02\n\x05Image\x12\x13\n\x0burlListList\x18\x01 \x03(\t\x12\x0b\n\x03uri\x18\x02 \x01(\t\x12\x0e\n\x06height\x18\x03 \x01(\x04\x12\r\n\x05width\x18\x04 \x01(\x04\x12\x10\n\x08\x61vgColor\x18\x05 \x01(\t\x12\x11\n\timageType\x18\x06 \x01(\r\x12\x12\n\nopenWebUrl\x18\x07 \x01(\t\x12*\n\x07\x63ontent\x18\x08 \x01(\x0b\x32\x19.douyin_util.ImageContent\x12\x12\n\nisAnimated\x18\t \x01(\x08\x12\x36\n\x0f\x46lexSettingList\x18\n \x01(\x0b\x32\x1d.douyin_util.NinePatchSetting\x12\x36\n\x0fTextSettingList\x18\x0b \x01(\x0b\x32\x1d.douyin_util.NinePatchSetting\"+\n\x10NinePatchSetting\x12\x17\n\x0fsettingListList\x18\x01 \x03(\t\"W\n\x0cImageContent\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tfontColor\x18\x02 \x01(\t\x12\r\n\x05level\x18\x03 \x01(\x04\x12\x17\n\x0f\x61lternativeText\x18\x04 \x01(\t\"\xb8\x01\n\tPushFrame\x12\r\n\x05seqId\x18\x01 \x01(\x04\x12\r\n\x05logId\x18\x02 \x01(\x04\x12\x0f\n\x07service\x18\x03 \x01(\x04\x12\x0e\n\x06method\x18\x04 \x01(\x04\x12-\n\x0bheadersList\x18\x05 \x03(\x0b\x32\x18.douyin_util.HeadersList\x12\x17\n\x0fpayloadEncoding\x18\x06 \x01(\t\x12\x13\n\x0bpayloadType\x18\x07 \x01(\t\x12\x0f\n\x07payload\x18\x08 \x01(\x0c\")\n\x0bHeadersList\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t*C\n\x0e\x43ommentTypeTag\x12\x19\n\x15\x43OMMENTTYPETAGUNKNOWN\x10\x00\x12\x16\n\x12\x43OMMENTTYPETAGSTAR\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'dy_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _RESPONSE_ROUTEPARAMSENTRY._options = None
  _RESPONSE_ROUTEPARAMSENTRY._serialized_options = b'8\001'
  _EFFECTCONFIG_EXTRAMAPENTRY._options = None
  _EFFECTCONFIG_EXTRAMAPENTRY._serialized_options = b'8\001'
  _globals['_COMMENTTYPETAG']._serialized_start=9913
  _globals['_COMMENTTYPETAG']._serialized_end=9980
  _globals['_RESPONSE']._serialized_start=26
  _globals['_RESPONSE']._serialized_end=392
  _globals['_RESPONSE_ROUTEPARAMSENTRY']._serialized_start=342
  _globals['_RESPONSE_ROUTEPARAMSENTRY']._serialized_end=392
  _globals['_MESSAGE']._serialized_start=395
  _globals['_MESSAGE']._serialized_end=549
  _globals['_CHATMESSAGE']._serialized_start=552
  _globals['_CHATMESSAGE']._serialized_end=1178
  _globals['_LANDSCAPEAREACOMMON']._serialized_start=1181
  _globals['_LANDSCAPEAREACOMMON']._serialized_end=1347
  _globals['_ROOMUSERSEQMESSAGE']._serialized_start=1350
  _globals['_ROOMUSERSEQMESSAGE']._serialized_end=1756
  _globals['_COMMONTEXTMESSAGE']._serialized_start=1758
  _globals['_COMMONTEXTMESSAGE']._serialized_end=1862
  _globals['_UPDATEFANTICKETMESSAGE']._serialized_start=1865
  _globals['_UPDATEFANTICKETMESSAGE']._serialized_end=2007
  _globals['_ROOMUSERSEQMESSAGECONTRIBUTOR']._serialized_start=2010
  _globals['_ROOMUSERSEQMESSAGECONTRIBUTOR']._serialized_end=2184
  _globals['_GIFTMESSAGE']._serialized_start=2187
  _globals['_GIFTMESSAGE']._serialized_end=3044
  _globals['_GIFTSTRUCT']._serialized_start=3047
  _globals['_GIFTSTRUCT']._serialized_end=3481
  _globals['_GIFTIMPRIORITY']._serialized_start=3483
  _globals['_GIFTIMPRIORITY']._serialized_end=3568
  _globals['_TEXTEFFECT']._serialized_start=3570
  _globals['_TEXTEFFECT']._serialized_end=3681
  _globals['_TEXTEFFECTDETAIL']._serialized_start=3684
  _globals['_TEXTEFFECTDETAIL']._serialized_end=4004
  _globals['_MEMBERMESSAGE']._serialized_start=4007
  _globals['_MEMBERMESSAGE']._serialized_end=4675
  _globals['_PUBLICAREACOMMON']._serialized_start=4677
  _globals['_PUBLICAREACOMMON']._serialized_end=4792
  _globals['_EFFECTCONFIG']._serialized_start=4795
  _globals['_EFFECTCONFIG']._serialized_end=5497
  _globals['_EFFECTCONFIG_EXTRAMAPENTRY']._serialized_start=5450
  _globals['_EFFECTCONFIG_EXTRAMAPENTRY']._serialized_end=5497
  _globals['_TEXT']._serialized_start=5500
  _globals['_TEXT']._serialized_end=5634
  _globals['_TEXTPIECE']._serialized_start=5637
  _globals['_TEXTPIECE']._serialized_end=5975
  _globals['_TEXTPIECEIMAGE']._serialized_start=5977
  _globals['_TEXTPIECEIMAGE']._serialized_end=6049
  _globals['_TEXTPIECEPATTERNREF']._serialized_start=6051
  _globals['_TEXTPIECEPATTERNREF']._serialized_end=6109
  _globals['_TEXTPIECEHEART']._serialized_start=6111
  _globals['_TEXTPIECEHEART']._serialized_end=6142
  _globals['_TEXTPIECEGIFT']._serialized_start=6144
  _globals['_TEXTPIECEGIFT']._serialized_end=6217
  _globals['_PATTERNREF']._serialized_start=6219
  _globals['_PATTERNREF']._serialized_end=6268
  _globals['_TEXTPIECEUSER']._serialized_start=6270
  _globals['_TEXTPIECEUSER']._serialized_end=6337
  _globals['_TEXTFORMAT']._serialized_start=6340
  _globals['_TEXTFORMAT']._serialized_end=6503
  _globals['_LIKEMESSAGE']._serialized_start=6506
  _globals['_LIKEMESSAGE']._serialized_end=6861
  _globals['_SOCIALMESSAGE']._serialized_start=6864
  _globals['_SOCIALMESSAGE']._serialized_end=7083
  _globals['_PICODISPLAYINFO']._serialized_start=7085
  _globals['_PICODISPLAYINFO']._serialized_end=7198
  _globals['_DOUBLELIKEDETAIL']._serialized_start=7200
  _globals['_DOUBLELIKEDETAIL']._serialized_end=7295
  _globals['_DISPLAYCONTROLINFO']._serialized_start=7297
  _globals['_DISPLAYCONTROLINFO']._serialized_end=7354
  _globals['_EPISODECHATMESSAGE']._serialized_start=7357
  _globals['_EPISODECHATMESSAGE']._serialized_end=7572
  _globals['_MATCHAGAINSTSCOREMESSAGE']._serialized_start=7575
  _globals['_MATCHAGAINSTSCOREMESSAGE']._serialized_end=7721
  _globals['_AGAINST']._serialized_start=7724
  _globals['_AGAINST']._serialized_end=8136
  _globals['_COMMON']._serialized_start=8139
  _globals['_COMMON']._serialized_end=8609
  _globals['_USER']._serialized_start=8612
  _globals['_USER']._serialized_end=9239
  _globals['_IMAGE']._serialized_start=9242
  _globals['_IMAGE']._serialized_end=9547
  _globals['_NINEPATCHSETTING']._serialized_start=9549
  _globals['_NINEPATCHSETTING']._serialized_end=9592
  _globals['_IMAGECONTENT']._serialized_start=9594
  _globals['_IMAGECONTENT']._serialized_end=9681
  _globals['_PUSHFRAME']._serialized_start=9684
  _globals['_PUSHFRAME']._serialized_end=9868
  _globals['_HEADERSLIST']._serialized_start=9870
  _globals['_HEADERSLIST']._serialized_end=9911
# @@protoc_insertion_point(module_scope)



def vn(val):
    if val < 0:
        raise ValueError
    buf = b""
    while val >> 7:
        m = val & 0xFF | 0x80
        buf += m.to_bytes(1, "big")
        val >>= 7
    buf += val.to_bytes(1, "big")
    return buf


def tp(a, b, ary):
    return vn((b << 3) | a) + ary


def rs(a, ary):
    if isinstance(ary, str):
        ary = ary.encode()
    return tp(2, a, vn(len(ary)) + ary)


def nm(a, ary):
    return tp(0, a, vn(ary))



import logging
import re
from struct import pack

import aiohttp

from biliup.plugins import match1

logger = logging.getLogger('biliup')


class Douyu:
    wss_url = 'wss://danmuproxy.douyu.com:8506/'
    heartbeat = b"\x14\x00\x00\x00\x14\x00\x00\x00\xb1\x02\x00\x00\x74\x79\x70\x65\x40\x3d\x6d\x72\x6b\x6c\x2f\x00"
    heartbeatInterval = 30
    msg_col = {'0': '16777215', '1': '16717077', '2': '2000880', '3': '8046667', '4': '16744192',
               '5': '10172916',
               '6': '16738740', '7': '16777215'}

    @staticmethod
    async def get_ws_info(url, context):
        async with aiohttp.ClientSession() as session:
            if 'm.douyu.com' in url:
                room_id = url.split('m.douyu.com/')[1].split('/')[0].split('?')[0]
            else:
                async with session.get(url, timeout=5) as resp:
                    room_page = await resp.text()
                    room_id = match1(room_page, r'\$ROOM\.room_id\s*=\s*(\d+)', r'apm_room_id\s*=\s*(\d+)')[0]
        reg_datas = []
        data = f'type@=loginreq/roomid@={room_id}/'
        s = pack('i', 9 + len(data)) * 2
        s += b'\xb1\x02\x00\x00'  # 689
        s += data.encode('ascii') + b'\x00'
        reg_datas.append(s)
        data = f'type@=joingroup/rid@={room_id}/gid@=-9999/'
        s = pack('i', 9 + len(data)) * 2
        s += b'\xb1\x02\x00\x00'  # 689
        s += data.encode('ascii') + b'\x00'
        reg_datas.append(s)
        return Douyu.wss_url, reg_datas

    @staticmethod
    def decode_msg(data):
        def stt_loads(stt_str):
            if '/' in stt_str:
                stt_items = stt_str.split('/')
                stt_list = []
                stt_dict = {}
                for stt_item in stt_items:
                    if stt_item == '':
                        continue
                    stt_item_decode = stt_loads(stt_item)
                    if type(stt_item_decode) is dict:
                        stt_dict.update(stt_item_decode)
                    else:
                        stt_list.append(stt_item_decode)
                if len(stt_list) > 0:
                    return stt_list
                else:
                    return stt_dict
            elif '@=' in stt_str:
                key, value = stt_str.split('@=')
                return {stt_loads(key): stt_loads(value)}
            else:
                return stt_str.replace("@A", "@").replace('@S', '/')

        msgs = []
        for msg in re.findall(b'(type@=.*?)\x00', data):
            try:
                msg = stt_loads(msg.decode('utf-8'))
                if type(msg) is dict:
                    msgs.append({
                        'name': msg.get('nn', ''),
                        'content': msg.get('txt', ''),
                        'msg_type': {
                            'dgb': 'gift',
                            'chatmsg': 'danmaku',
                            'uenter': 'enter'
                        }.get(msg['type'], 'other'),
                        'color': Douyu.msg_col.get(msg.get('col')),
                    })
            except:
                logger.warning(f"{Douyu.__name__}: 弹幕接收异常", exc_info=True)
        return msgs



import functools
import importlib
import pkgutil
import re


def suit_url(pattern, urls):
    sorted_url = []
    for i in range(len(urls) - 1, -1, -1):
        if re.match(pattern, urls[i]):
            sorted_url.append(urls[i])
            urls.remove(urls[i])
    return sorted_url


class Plugin:
    download_plugins = []
    upload_plugins = {}

    def __init__(self, pkg):
        self.load_plugins(pkg)

    @staticmethod
    def download(regexp):
        def decorator(cls):
            cls.VALID_URL_BASE = regexp
            Plugin.download_plugins.append(cls)
            return cls
        return decorator

    @staticmethod
    def upload(platform):
        def decorator(cls):
            @functools.wraps(cls)
            def wrapper(*args, **kw):
                print(f"args {args}")
                print(f"kw {kw}")
                return cls(*args, **kw)
            Plugin.upload_plugins[platform] = wrapper
            return wrapper
        return decorator

    @classmethod
    def sorted_checker(cls, urls):
        if not urls:
            return {}
        from ..plugins import general
        curls = urls.copy()
        checker_plugins = {}
        for plugin in cls.download_plugins:
            url_list = suit_url(plugin.VALID_URL_BASE, curls)
            if not url_list:
                continue
            else:
                plugin.url_list = url_list
                checker_plugins[plugin.__name__] = plugin
            if not curls:
                return checker_plugins
        general.__plugin__.url_list = curls
        checker_plugins[general.__plugin__.__name__] = general.__plugin__
        return checker_plugins

    @classmethod
    def inspect_checker(cls, url):
        from ..plugins import general
        for plugin in cls.download_plugins:
            if not re.match(plugin.VALID_URL_BASE, url):
                continue
            else:
                return plugin
        return general.__plugin__

    def load_plugins(self, pkg):
        """Attempt to load plugins from the path specified.
        engine.plugins.__path__[0]: full path to a directory where to look for plugins
        """

        plugins = []

        for loader, name, ispkg in pkgutil.iter_modules([pkg.__path__[0]]):
            # set the full plugin module name
            module_name = f"{pkg.__name__}.{name}"
            module = importlib.import_module(module_name)
            if ispkg:
                self.load_plugins(module)
                continue
            if module in plugins:
                continue
            plugins.append(module)
        return plugins



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# filename: __servantproxy.py

# Tencent is pleased to support the open source community by making Tars available.
#
# Copyright (C) 2016THL A29 Limited, a Tencent company. All rights reserved.
#
# Licensed under the BSD 3-Clause License (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# https://opensource.org/licenses/BSD-3-Clause
#
# Unless required by applicable law or agreed to in writing, software distributed
# under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied. See the License for the
# specific language governing permissions and limitations under the License.
#


'''
@version: 0.01
@brief: rpc抽离出servantproxy
'''
import threading
import time

# from __packet import ResponsePacket
from biliup.Danmaku.tars.__TimeoutQueue import ReqMessage
from biliup.Danmaku.tars.__logger import tarsLogger
from biliup.Danmaku.tars.__packet import RequestPacket
from biliup.Danmaku.tars.__util import util
from biliup.Danmaku.tars import exception
from biliup.Danmaku.tars.exception import TarsException


class ServantProxy(object):
    '''
    @brief: 1、远程对象的本地代理
            2、同名servant在一个通信器中最多只有一个实例
            3、防止和用户在Tars中定义的函数名冲突，接口以tars_开头
    '''

    # 服务器响应的错误码
    TARSSERVERSUCCESS = 0  # 服务器端处理成功
    TARSSERVERDECODEERR = -1  # 服务器端解码异常
    TARSSERVERENCODEERR = -2  # 服务器端编码异常
    TARSSERVERNOFUNCERR = -3  # 服务器端没有该函数
    TARSSERVERNOSERVANTERR = -4  # 服务器端五该Servant对象
    TARSSERVERRESETGRID = -5  # 服务器端灰度状态不一致
    TARSSERVERQUEUETIMEOUT = -6  # 服务器队列超过限制
    TARSASYNCCALLTIMEOUT = -7  # 异步调用超时
    TARSPROXYCONNECTERR = -8  # proxy链接异常
    TARSSERVERUNKNOWNERR = -99  # 服务器端未知异常

    TARSVERSION = 1
    TUPVERSION = 2
    TUPVERSION2 = 3

    TARSNORMAL = 0
    TARSONEWAY = 1

    TARSMESSAGETYPENULL = 0
    TARSMESSAGETYPEHASH = 1
    TARSMESSAGETYPEGRID = 2
    TARSMESSAGETYPEDYED = 4
    TARSMESSAGETYPESAMPLE = 8
    TARSMESSAGETYPEASYNC = 16

    mapcls_context = util.mapclass(util.string, util.string)

    def __init__(self):
        tarsLogger.debug('ServantProxy:__init__')
        self.__reactor = None
        self.__object = None
        self.__initialize = False

    def __del__(self):
        tarsLogger.debug('ServantProxy:__del__')

    def _initialize(self, reactor, obj):
        '''
        @brief: 初始化函数，需要调用才能使用ServantProxy
        @param reactor: 网络管理的reactor实例
        @type reactor: FDReactor
        @return: None
        @rtype: None
        '''
        tarsLogger.debug('ServantProxy:_initialize')

        assert(reactor and obj)
        if self.__initialize:
            return
        self.__reactor = reactor
        self.__object = obj
        self.__initialize = True

    def _terminate(self):
        '''
        @brief: 不再使用ServantProxy时调用，会释放相应资源
        @return: None
        @rtype: None
        '''
        tarsLogger.debug('ServantProxy:_terminate')
        self.__object = None
        self.__reactor = None
        self.__initialize = False

    def tars_name(self):
        '''
        @brief: 获取ServantProxy的名字
        @return: ServantProxy的名字
        @rtype: str
        '''
        return self.__object.name()

    def tars_timeout(self):
        '''
        @brief: 获取超时时间，单位是ms
        @return: 超时时间
        @rtype: int
        '''
        # 默认的为3S = ObjectProxy.DEFAULT_TIMEOUT
        return int(self.__timeout() * 1000)

    def tars_ping(self):
        pass

    # def tars_initialize(self):
        # pass

    # def tars_terminate(self):
        # pass

    def tars_invoke(self, cPacketType, sFuncName, sBuffer, context, status):
        '''
        @brief: TARS协议同步方法调用
        @param cPacketType: 请求包类型
        @type cPacketType: int
        @param sFuncName: 调用函数名
        @type sFuncName: str
        @param sBuffer: 序列化后的发送参数
        @type sBuffer: str
        @param context: 上下文件信息
        @type context: ServantProxy.mapcls_context
        @param status: 状态信息
        @type status:
        @return: 响应报文
        @rtype: ResponsePacket
        '''
        tarsLogger.debug('ServantProxy:tars_invoke, func: %s', sFuncName)
        req = RequestPacket()
        req.iVersion = ServantProxy.TARSVERSION
        req.cPacketType = cPacketType
        req.iMessageType = ServantProxy.TARSMESSAGETYPENULL
        req.iRequestId = 0
        req.sServantName = self.tars_name()
        req.sFuncName = sFuncName
        req.sBuffer = sBuffer
        req.iTimeout = self.tars_timeout()

        reqmsg = ReqMessage()
        reqmsg.type = ReqMessage.SYNC_CALL
        reqmsg.servant = self
        reqmsg.lock = threading.Condition()
        reqmsg.request = req
        reqmsg.begtime = time.time()
        # # test
        reqmsg.isHash = True
        reqmsg.isConHash = True
        reqmsg.hashCode = 123456

        rsp = None
        try:
            rsp = self.__invoke(reqmsg)
        except exception.TarsSyncCallTimeoutException:
            if reqmsg.adapter:
                reqmsg.adapter.finishInvoke(True)
            raise
        except TarsException:
            raise
        except:
            raise TarsException('ServantProxy::tars_invoke excpetion')

        if reqmsg.adapter:
            reqmsg.adapter.finishInvoke(False)

        return rsp

    def tars_invoke_async(self, cPacketType, sFuncName, sBuffer,
                          context, status, callback):
        '''
        @brief: TARS协议同步方法调用
        @param cPacketType: 请求包类型
        @type cPacketType: int
        @param sFuncName: 调用函数名
        @type sFuncName: str
        @param sBuffer: 序列化后的发送参数
        @type sBuffer: str
        @param context: 上下文件信息
        @type context: ServantProxy.mapcls_context
        @param status: 状态信息
        @type status:
        @param callback: 异步调用回调对象
        @type callback: ServantProxyCallback的子类
        @return: 响应报文
        @rtype: ResponsePacket
        '''
        tarsLogger.debug('ServantProxy:tars_invoke')
        req = RequestPacket()
        req.iVersion = ServantProxy.TARSVERSION
        req.cPacketType = cPacketType if callback else ServantProxy.TARSONEWAY
        req.iMessageType = ServantProxy.TARSMESSAGETYPENULL
        req.iRequestId = 0
        req.sServantName = self.tars_name()
        req.sFuncName = sFuncName
        req.sBuffer = sBuffer
        req.iTimeout = self.tars_timeout()

        reqmsg = ReqMessage()
        reqmsg.type = ReqMessage.ASYNC_CALL if callback else ReqMessage.ONE_WAY
        reqmsg.callback = callback
        reqmsg.servant = self
        reqmsg.request = req
        reqmsg.begtime = time.time()

        rsp = None
        try:
            rsp = self.__invoke(reqmsg)
        except TarsException:
            raise
        except Exception:
            raise TarsException('ServantProxy::tars_invoke excpetion')

        if reqmsg.adapter:
            reqmsg.adapter.finishInvoke(False)

        return rsp

    def __timeout(self):
        '''
        @brief: 获取超时时间，单位是s
        @return: 超时时间
        @rtype: float
        '''
        return self.__object.timeout()

    def __invoke(self, reqmsg):
        '''
        @brief: 远程过程调用
        @param reqmsg: 请求数据
        @type reqmsg: ReqMessage
        @return: 调用成功或失败
        @rtype: bool
        '''
        tarsLogger.debug('ServantProxy:invoke, func: %s',
                         reqmsg.request.sFuncName)
        ret = self.__object.invoke(reqmsg)
        if ret == -2:
            errmsg = ('ServantProxy::invoke fail, no valid servant,' +
                      ' servant name : %s, function name : %s' %
                      (reqmsg.request.sServantName,
                       reqmsg.request.sFuncName))
            raise TarsException(errmsg)
        if ret == -1:
            errmsg = ('ServantProxy::invoke connect fail,' +
                      ' servant name : %s, function name : %s, adapter : %s' %
                      (reqmsg.request.sServantName,
                       reqmsg.request.sFuncName,
                       reqmsg.adapter.getEndPointInfo()))
            raise TarsException(errmsg)
        elif ret != 0:
            errmsg = ('ServantProxy::invoke unknown fail, ' +
                      'Servant name : %s, function name : %s' %
                      (reqmsg.request.sServantName,
                       reqmsg.request.sFuncName))
            raise TarsException(errmsg)

        if reqmsg.type == ReqMessage.SYNC_CALL:
            reqmsg.lock.acquire()
            reqmsg.lock.wait(self.__timeout())
            reqmsg.lock.release()

            if not reqmsg.response:
                errmsg = ('ServantProxy::invoke timeout: %d, servant name'
                          ': %s, adapter: %s, request id: %d' % (
                              self.tars_timeout(),
                              self.tars_name(),
                              reqmsg.adapter.trans().getEndPointInfo(),
                              reqmsg.request.iRequestId))
                raise exception.TarsSyncCallTimeoutException(errmsg)
            elif reqmsg.response.iRet == ServantProxy.TARSSERVERSUCCESS:
                return reqmsg.response
            else:
                errmsg = 'servant name: %s, function name: %s' % (
                         self.tars_name(), reqmsg.request.sFuncName)
                self.tarsRaiseException(reqmsg.response.iRet, errmsg)

    def _finished(self, reqmsg):
        '''
        @brief: 通知远程过程调用线程响应报文到了
        @param reqmsg: 请求响应报文
        @type reqmsg: ReqMessage
        @return: 函数执行成功或失败
        @rtype: bool
        '''
        tarsLogger.debug('ServantProxy:finished')
        if not reqmsg.lock:
            return False
        reqmsg.lock.acquire()
        reqmsg.lock.notifyAll()
        reqmsg.lock.release()
        return True

    def tarsRaiseException(self, errno, desc):
        '''
        @brief: 服务器调用失败，根据服务端给的错误码抛出异常
        @param errno: 错误码
        @type errno: int
        @param desc: 错误描述
        @type desc: str
        @return: 没有返回值，函数会抛出异常
        @rtype:
        '''
        if errno == ServantProxy.TARSSERVERSUCCESS:
            return

        elif errno == ServantProxy.TARSSERVERDECODEERR:
            raise exception.TarsServerDecodeException(
                "server decode exception: errno: %d, msg: %s" % (errno, desc))

        elif errno == ServantProxy.TARSSERVERENCODEERR:
            raise exception.TarsServerEncodeException(
                "server encode exception: errno: %d, msg: %s" % (errno, desc))

        elif errno == ServantProxy.TARSSERVERNOFUNCERR:
            raise exception.TarsServerNoFuncException(
                "server function mismatch exception: errno: %d, msg: %s" % (errno, desc))

        elif errno == ServantProxy.TARSSERVERNOSERVANTERR:
            raise exception.TarsServerNoServantException(
                "server servant mismatch exception: errno: %d, msg: %s" % (errno, desc))

        elif errno == ServantProxy.TARSSERVERRESETGRID:
            raise exception.TarsServerResetGridException(
                "server reset grid exception: errno: %d, msg: %s" % (errno, desc))

        elif errno == ServantProxy.TARSSERVERQUEUETIMEOUT:
            raise exception.TarsServerQueueTimeoutException(
                "server queue timeout exception: errno: %d, msg: %s" % (errno, desc))

        elif errno == ServantProxy.TARSPROXYCONNECTERR:
            raise exception.TarsServerQueueTimeoutException(
                "server connection lost: errno: %d, msg: %s" % (errno, desc))

        else:
            raise exception.TarsServerUnknownException(
                "server unknown exception: errno: %d, msg: %s" % (errno, desc))



import logging
import platform
import sys

__version__ = "0.4.72"

LOG_CONF = {
    'version': 1,
    'formatters': {
        'verbose': {
            'format': "%(asctime)s %(filename)s[line:%(lineno)d](Pid:%(process)d "
                      "Tname:%(threadName)s) %(levelname)s %(message)s",
            # 'datefmt': "%Y-%m-%d %H:%M:%S"
        },
        'simple': {
            'format': '%(asctime)s %(filename)s%(lineno)d[%(levelname)s]Tname:%(threadName)s %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': logging.DEBUG,
            'class': 'logging.StreamHandler',
            'stream': sys.stdout,
            'formatter': 'simple'
        },
        'file': {
            'level': logging.DEBUG,
            'class': 'biliup.common.log.SafeRotatingFileHandler',
            'when': 'W0',
            'interval': 1,
            'backupCount': 1,
            'filename': 'ds_update.log',
            'formatter': 'verbose',
            'encoding': 'utf-8'
        }
    },
    'root': {
        'handlers': ['console'],
        'level': logging.INFO,
    },
    'loggers': {
        'biliup': {
            'handlers': ['file'],
            'level': logging.INFO,
        },
    }
}

if (3, 10, 6) > sys.version_info >= (3, 8) and platform.system() == 'Windows':
    # fix 'Event loop is closed' RuntimeError in Windows
    from asyncio import proactor_events
    from biliup.common.tools import silence_event_loop_closed

    proactor_events._ProactorBasePipeTransport.__del__ = silence_event_loop_closed(
        proactor_events._ProactorBasePipeTransport.__del__)


if getattr(sys, 'frozen', False) and hasattr(sys, '_MEIPASS'):
    import multiprocessing
    multiprocessing.freeze_support()
    print('running in a PyInstaller bundle')


import contextvars
import os
import time
from datetime import datetime, timedelta
from pathlib import Path
from typing import List

from sqlalchemy import select, desc, delete
from sqlalchemy.orm import sessionmaker, scoped_session, Session
from alembic import command, config

from .models import (
    DB_PATH,
    engine,
    BaseModel,
    StreamerInfo,
    FileList,
)

SessionLocal = sessionmaker(bind=engine, autocommit=False)
# 使用 Context ID 区分会话
# Session = scoped_session(session_factory, scopefunc=lambda: id(contextvars.copy_context()))


def struct_time_to_datetime(date: time.struct_time):
    return datetime.fromtimestamp(time.mktime(date))


def datetime_to_struct_time(date: datetime):
    return time.localtime(date.timestamp())


def init(no_http, from_config):
    """初始化数据库"""
    first_run = not Path.cwd().joinpath("data/data.sqlite3").exists()
    if no_http and not first_run and from_config:
        new_name = f'{DB_PATH}.backup'
        if os.path.exists(new_name):
            os.remove(new_name)
        os.rename(DB_PATH, new_name)
        print(f"旧数据库已备份为: {new_name}")  # 在logger加载配置之前执行，只能使用print
    BaseModel.metadata.create_all(engine)  # 创建所有表
    migrate_via_alembic()
    return first_run or no_http


def get_stream_info(db: Session, name: str) -> dict:
    """根据 streamer 获取下载信息, 若不存在则返回空字典"""
    res = db.execute(
        select(StreamerInfo).
        filter_by(name=name).
        order_by(desc(StreamerInfo.id))
    ).first()
    if res:
        res = res._asdict()
        res["date"] = datetime_to_struct_time(res["date"])
        return res
    return {}


def get_stream_info_by_filename(db: Session, filename: str) -> dict:
    """通过文件名获取下载信息, 若不存在则返回空字典"""
    try:
        # stream_info = FileList.get(FileList.file == filename).streamer_info
        stream_info = db.execute(
            select(FileList).
            where(FileList.file == filename)
        ).scalar_one().streamerinfo
        stream_info_dict = stream_info.as_dict()
    except Exception:
        return {}
    stream_info_dict = {key: value for key, value in stream_info_dict.items() if value}  # 清除字典中的空元素
    stream_info_dict["date"] = datetime_to_struct_time(stream_info_dict["date"])  # 将开播时间转回 struct_time 类型
    return stream_info_dict


def add_stream_info(db: Session, name: str, url: str, date: time.struct_time) -> int:
    """添加下载信息, 返回所添加行的 id """
    streamerinfo = StreamerInfo(
        name=name,
        url=url,
        date=struct_time_to_datetime(date),
        title="",
        live_cover_path="",
    )
    db.add(streamerinfo)
    db.commit()
    return streamerinfo.id


def delete_stream_info(db: Session, name: str) -> int:
    """根据 streamer 删除下载信息, 返回删除的行数, 若不存在则返回 0 """
    result = db.execute(
        delete(StreamerInfo).where(StreamerInfo.name == name))
    db.commit()
    # db.refresh(result)
    return result.rowcount()


def delete_stream_info_by_date(db: Session, name: str, date: time.struct_time) -> int:
    """根据 streamer 和开播时间删除下载信息, 返回删除的行数, 若不存在则返回 0 """
    start_datetime = struct_time_to_datetime(date)
    stmt = delete(StreamerInfo).where(
        (StreamerInfo.name == name),
        StreamerInfo.date.between(
            start_datetime - timedelta(minutes=1),
            start_datetime + timedelta(minutes=1)),
    )
    result = db.execute(stmt)
    db.commit()
    return result.rowcount()


def update_cover_path(db: Session, database_row_id: int, live_cover_path: str):
    """更新封面存储路径"""
    if not live_cover_path:
        live_cover_path = ""
    streamerinfo = db.scalar(select(StreamerInfo).where(StreamerInfo.id == database_row_id))
    streamerinfo.live_cover_path = live_cover_path
    db.commit()


def update_room_title(db: Session, database_row_id: int, title: str):
    """更新直播标题"""
    if not title:
        title = ""
    streamerinfo = db.get(StreamerInfo, database_row_id)
    streamerinfo.title = title
    db.commit()


def update_file_list(db: Session, database_row_id: int, file_name: str) -> int:
    """向视频文件列表中添加文件名"""
    streamer_info = db.get(StreamerInfo, database_row_id)
    file_list = FileList(file=file_name, streamer_info_id=streamer_info.id)
    db.add(file_list)
    db.commit()
    return file_list.id


# def delete_file_list(db: Session, database_row_id: int, file_name: str) -> int:
#     """从视频文件列表中删除指定的文件名，返回删除的行数，若不存在则返回 0"""
#     # 查询数据库以获取对应的streamer_info
#     streamer_info = db.get(StreamerInfo, database_row_id)
#     if not streamer_info:
#         return 0
#     stmt = delete(FileList).where(
#         (FileList.file == file_name),
#         (FileList.streamer_info_id == streamer_info.id)
#     )
#     result = db.execute(stmt)
#     db.commit()
#     return result.rowcount


def get_file_list(db: Session, database_row_id: int) -> List[str]:
    """获取视频文件列表"""
    file_list = db.get(StreamerInfo, database_row_id).filelist
    return [file.file for file in file_list]


def migrate_via_alembic():
    """ 自动迁移，通过 alembic 实现 """
    def process_revision_directives(context, revision, directives):
        """ 如果无改变，不生成迁移脚本 """
        script = directives[0]
        if script.upgrade_ops.is_empty():
            directives[:] = []
    alembic_cfg = config.Config()
    # 获取脚本路径
    script_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "migration")
    versions_scripts_path = os.path.join(script_path, 'versions')
    if not os.path.exists(versions_scripts_path):
        os.makedirs(versions_scripts_path, 0o700)
    alembic_cfg.set_main_option('script_location', script_path)
    command.stamp(alembic_cfg, 'head', purge=True)  # 将当前标记为最新版
    scripts = command.revision(  # 自动生成迁移脚本
        alembic_cfg,
        autogenerate=True,
        process_revision_directives=process_revision_directives
    )
    if not scripts:
        print("数据库已是最新版本")
        return
    command.upgrade(alembic_cfg, 'head')
    print("检测到旧版数据库，已完成自动迁移")

def backup(db: Session):
    """备份数据库"""
    pass
